###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: dhcp.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: dhcp
# version: 0.2.0.0
# config: dhcp.conf
# description:
# requires: this_module_needs_to_be_converted

###############################################################################
##
##  egg_dhcp - demo Tcl for eggdrop to get it's ip and hostname
##
###############################################################################
##
## Example use:
##
## source egg_dhcp-0.1.tcl
##
## set my-ip [egg_dhcp:get_ip eth0]
## set my-hostname [egg_dhcp:get_hostname ${my-ip}]
##
###############################################################################

#
# Default settings:
#
array set egg_dhcp_settings {
  ifconfig /sbin/ifconfig
  host /usr/bin/host
}

#
# egg_dhcp:get_ip interface
#
proc egg_dhcp:get_ip {interface} {
  global egg_dhcp_settings
  # 'ifconfig eth0'
  # eth0      Link encap:Ethernet  HWaddr 00:00:00:00:00:00
  #           inet addr:192.168.0.1  Bcast:192.168.0.255  Mask:255.255.255.0
  #           UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
  #           RX packets:0 errors:0 dropped:0 overruns:0 frame:0
  #           TX packets:0 errors:0 dropped:0 overruns:0 carrier:0
  #           collisions:0
  #           Interrupt:0 Base address:0x000
  set data [exec $egg_dhcp_settings(ifconfig) $interface]
  while {[regsub -all -- "\ \ " $data "\ " data]} {}
  foreach i [split $data \n] {
    if {[regexp " inet addr:.*" $i]} then {
      return [lindex [split $i ":\ "] 3]
    }
  }
  return
}

#
# egg_dhcp:get_hostname ip
#
if {[string compare "" [info commands dnslookup]]} then {
  proc egg_dhcp:get_hostname {ip} {
    return [dnslookup $ip]
  }
} else {
  proc egg_dhcp:get_hostname {ip} {
    global egg_dhcp_settings
    # 'host 192.168.0.1'
    # 1.0.168.192.IN-ADDR.ARPA domain name pointer foo.bar.org
    return [lindex [exec $egg_dhcp_settings(host) $ip] 4]
  }
}

return
