###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: ircaway.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: ircaway
# version: 1.4.0.0
# description: Provides dcc, msg, and secure public ircaway commands that \
#	can be used to set the bot /away.
# requires: authCheck commandChar put

namespace eval ircaway {

  ##
  ## Set away status: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1
  ##
  proc pub:ircaway {nick uhost hand chan arg} {
    return [ircaway pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  ##
  ## Set away status: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1
  ##
  proc msg:ircaway {nick uhost hand arg} {
    return [ircaway msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Set away status: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:ircaway {hand idx arg} {
    return [ircaway dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  ##
  ## Set away status
  ##
  ## Args:    type, resp1, dest1, resp2, dest2,
  ##          nick, user@host, handle, channel, arg
  ## Returns: 1
  ##
  proc ircaway {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type ircaway $nick $uhost $hand $chan]} then {
      set reason [string trimright $arg]
      if {![string compare "" $reason]} then {
        putserv "AWAY :"
        put $resp1 $dest1 "Bot is no longer marked as away"
      } else {
        putserv "AWAY :$reason"
        put $resp1 $dest1 "Bot is now marked as away: $reason"
      }
      return 1
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc"
    }
    foreach type $what {
      switch -exact -- $type {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub m|- ${char}ircaway ${NamespaceCurrent}::pub:ircaway
        }
        msg {
          $action msg m|- ircaway ${NamespaceCurrent}::msg:ircaway
        }
        dcc {
          $action dcc m|- ircaway ${NamespaceCurrent}::dcc:ircaway
        }
      }
    }
    return
  }

} ;#namespace ircaway

return
