###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: servman.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

## Ideas from klined.tcl, restrict.tcl, and mainserver.tcl

# name: servman
# version: 0.8.0.0
# config: servman.conf
# description: Server list manager that adds dcc commands called `.+server', \
#	`.-server', and `.mvserver' to add/remove/move servers in the server \
#	list, adds dcc commands `.+kline', and `.-kline' to flag servers as \
#	having k-lines for the bot, replaces dcc commands `.servers' and \
#	`.jump' with enhanced versions, adds a `.klines' command to display \
#	k-line details for servers, and automatically adds servers to the \
#	k-line list when they have a k-line for the bot.
# requires: wpLog listMaxElementLength matchServer isNumber convertDHM \
#	formatTime convertSeconds

namespace eval servman {

  ##
  ## Show servers in the server list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:servers {hand idx arg} {
    global server
    variable serverdata
    variable klinedata

    if {[set fmtlen1 [string length [llength $serverdata]]]} then {
      set count 1
      set donehere 0
      set serverLower [string tolower $server]
      set fmtlen2 [listMaxElementLength $serverdata]
      putdcc $idx "My server list:"
      foreach serv $serverdata {
        set status ""
        if {(!$donehere) && \
            (![string compare $serverLower [string tolower $serv]])} then {
          append status " <- I am here"
          set donehere 1
        }
        if {[matchServer $klinedata $serv] >= 0} then {
          append status " (k-lined)"
        }
        putdcc $idx [format "  %-${fmtlen1}s %-${fmtlen2}s %s" $count $serv $status]
        incr count
      }
      if {(!$donehere) && ([string compare "" $server])} then {
        putdcc $idx [format "  %-${fmtlen1}s %-${fmtlen2}s %s" "" $server " <- I am here (removed)"]
      }
    } else {
      putdcc $idx "No servers."
    }
    return 1
  }

  ##
  ## Add a server to the server list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:+server {hand idx arg} {
    variable serverdata

    set arg [split [string trimright $arg]]
    set server [lindex $arg 0]
    set where [lindex $arg 1]
    if {![string compare "" $server]} then {
      putdcc $idx "Usage: +server <server> \[location\]"
    } elseif {[isNumber $where]} then {
      set index [expr $where - 1]
      if {($index >= 0) && ($index <= [llength $serverdata])} then {
        set serverdata [linsert $serverdata $index $server]
        putdcc $idx "Added server `$server' to server list at position `$where'."
      } else {
        putdcc $idx "Invalid server list position: $where"
      }
    } else {
      lappend serverdata $server
      putdcc $idx "Added server `$server' to server list."
    }
    return 1
  }

  ##
  ## Remove a server from the server list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:-server {hand idx arg} {
    variable serverdata

    set what [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $what]} then {
      putdcc $idx "Usage: -server <server|locaction>"
    } elseif {[isNumber $what]} then {
      set index [expr $what - 1]
      if {($index >= 0) && ($index <= [expr [llength $serverdata] - 1])} then {
        set server [lindex $serverdata $index]
        set serverdata [lreplace $serverdata $index $index]
        putdcc $idx "Removed server `$server' from server list."
      } else {
        putdcc $idx "Invalid server list position: $what"
      }
    } else {
      set index [matchServer $serverdata $what]
      switch -exact -- $index {
        -1 {
          putdcc $idx "No such server."
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $what"
        }
        default {
          set server [lindex $serverdata $index]
          set serverdata [lreplace $serverdata $index $index]
          putdcc $idx "Removed server `$server' from server list."
        }
      }
    }
    return 1
  }

  ##
  ## Reorder the server list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:mvserver {hand idx arg} {
    variable serverdata

    set arg [split [string trimright $arg]]
    set what [lindex $arg 0]
    set where [lindex $arg 1]
    if {(![string compare "" $what]) || (![string compare "" $where])} then {
      putdcc $idx "Usage: mvserver <server|locaction> <new name|new locaction>"
    } elseif {[isNumber $what]} then {
      set index [expr $what - 1]
      if {($index >= 0) && ($index <= [expr [llength $serverdata] - 1])} then {
        set server [lindex $serverdata $index]
      } else {
        putdcc $idx "Invalid server list position: $what"
      }
    } else {
      set index [matchServer $serverdata $what]
      switch -exact -- $index {
        -1 {
          putdcc $idx "No such server."
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $what"
        }
        default {
          set server [lindex $serverdata $index]
        }
      }
    }
    if {[info exist server]} then {
      if {[isNumber $where]} then {
        set index2 [expr $where - 1]
        if {($index2 >= 0) && ($index2 <= [expr [llength $serverdata] - 1])} then {
          set serverdata [linsert [lreplace $serverdata $index $index] $index2 $server]
          putdcc $idx "Moved server `$server' from server list position `[expr $index + 1]' to `$where'."
        } else {
          putdcc $idx "Invalid server list position: $where"
        }
      } else {
        set serverdata [lreplace $serverdata $index $index $where]
        putdcc $idx "Renamed server `$server' to `$where'."
      }
    }
    return 1
  }

  ##
  ## Show klined servers in the server list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:klines {hand idx arg} {
    variable serverdata
    variable klinedata

    if {[set length [llength $klinedata]]} then {
      set fmtlen [string length $length]
      putdcc $idx "My klined server list:"
      foreach list $klinedata {
        foreach {server added expire reason} $list {break}
        set index [expr [matchServer $serverdata $server] + 1]
        if {$index < 1} then {
          continue
        }
        if {[isNumber $added]} then {
          set addedtime [formatTime $added]
          if {[isNumber $expire]} then {
            set expiretime [formatTime [expr $added + $expire]]
          }
        }
        putdcc $idx [format "  %-${fmtlen}s %s" $index $server]
        if {[info exists addedtime]} then {
          putdcc $idx [format "  %-${fmtlen}s" "  Added:   $addedtime"]
        }
        if {[info exists expiretime]} then {
          putdcc $idx [format "  %-${fmtlen}s" "  Expires: $expiretime"]
        }
        if {[string compare "" $reason]} then {
          putdcc $idx [format "  %-${fmtlen}s" "  Reason:  $reason"]
        }
      }
    } else {
      putdcc $idx "No klined servers."
    }
    return 1
  }

  ##
  ## Mark a server as k-lined: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:+kline {hand idx arg} {
    variable serverdata
    variable klinedata

    set arg [split [string trimright $arg]]
    set what [lindex $arg 0]
    set tmp [lindex $arg 1]
    if {![string compare "" $what]} then {
      putdcc $idx "Usage: +kline <server|location> \[%expiretime <XdXhXm>\] \[reason\]"
    } elseif {![string compare % [string index $tmp 0]]} then {
      set expire [convertDHM [string trimleft $tmp %]]
      set reason [join [lrange $arg 2 end]]
    } else {
      set expire 0
      set reason [join [lrange $arg 1 end]]
    }
    set time [clock seconds]
    if {[isNumber $what]} then {
      set index [expr $what - 1]
      if {($index >= 0) && ($index <= [expr [llength $serverdata] - 1])} then {
        set server [lindex $serverdata $index]
      } else {
        putdcc $idx "Invalid server list position: $what"
      }
    } else {
      set index [matchServer $serverdata $what]
      switch -exact -- $index {
        -1 {
          putdcc $idx "No such server."
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $what"
        }
        default {
          set server [lindex $serverdata $index]
        }
      }
    }
    if {[info exists server]} then {
      switch -exact -- [matchServer $klinedata $server] {
        -1 {
          if {[string compare "" $reason]} then {
            lappend klinedata [list $server $time $expire $reason]
            putdcc $idx "Marked server `$server' as k-lined with reason: $reason"
          } else {
            lappend klinedata [list $server $time $expire]
            putdcc $idx "Marked server `$server' as k-lined."
          }
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $server"
        }
        default {
          putdcc $idx "That server is already maked as k-lined."
        }
      }
    }
    return 1
  }

  ##
  ## Mark a server as no longer k-lined: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:-kline {hand idx arg} {
    variable serverdata
    variable klinedata

    set what [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $what]} then {
      putdcc $idx "Usage: -kline <server|locaction>"
    } elseif {[isNumber $what]} then {
      set index [expr $what - 1]
      if {($index >= 0) && ($index <= [expr [llength $serverdata] - 1])} then {
        set server [lindex $serverdata $index]
      } else {
        putdcc $idx "Invalid server list position: $what"
      }
    } else {
      set index [matchServer $serverdata $what]
      switch -exact -- $index {
        -1 {
          putdcc $idx "No such server."
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $what"
        }
        default {
          set server [lindex $serverdata $index]
        }
      }
    }
    if {[info exists server]} then {
      set index2 [matchServer $klinedata $server]
      switch -exact -- $index2 {
        -1 {
          putdcc $idx "That server is not maked as k-lined."
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $server"
        }
        default {
          set klinedata [lreplace $klinedata $index2 $index2]
          putdcc $idx "Marked server `$server' as no longer k-lined."
        }
      }
    }
    return 1
  }

  ##
  ## Jump to the given server: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:jump {hand idx arg} {
    variable serverdata

    set arg [split [string trimright $arg]]
    set what [lindex $arg 0]
    set port [lindex $arg 1]
    set pass [lindex $arg 2]
    if {![string compare "" $what]} then {
      jump
    } elseif {[isNumber $what]} then {
      set index [expr $what - 1]
      if {($index >= 0) && ($index <= [expr [llength $serverdata] - 1])} then {
        foreach {server port pass} [split [lindex $serverdata $index] :] {break}
      } else {
        putdcc $idx "Invalid server list position: $what"
      }
    } else {
      set index [matchServer $serverdata $what]
      switch -exact -- $index {
        -1 {
          set server $what
        }
        -2 {
          putdcc $idx "Ambiguous match for server: $what"
        }
        default {
          foreach {server port pass} [split [lindex $serverdata $index] :] {break}
        }
      }
    }
    if {[info exists server]} then {
      eval jump [split $server] [split $port] [split $pass]
    }
    return 1
  }

  ##
  ## Detect nickname collision and attempt to reconnect to the same server
  ##
  ## 436    ERR_NICKCOLLISION
  ##        "<nick> :Nickname collision KILL"
  ##
  ## Args:    from, keyword, arg
  ## Returns: 0
  ##
  proc raw:436 {from key arg} {
    global botnick
    variable NamespaceCurrent

    if {[set index [matchServer $serverdata $from]] == -2} then {
      set index [matchServer $serverdata $from 0]
    }
    if {$index >= 0} then {
      set server [lindex $serverdata $index]
      wpLog s * $NamespaceCurrent "Error: Nickname collision with nickname `$botnick', attempting to reconnect to server `$from'..."
      eval jump [split $server :]
    } else {
      wpLog s * $NamespaceCurrent "Error: Nickname collision with nickname `$botnick', attempting to reconnect to a server..."
      jump
    }
    return 0
  }

  ##
  ## Detect and flag servers that have the bot k-lined
  ##
  ## 465    ERR_YOUREBANNEDCREEP
  ##        ":You are banned from this server"
  ##
  ## Args:    from, keyword, arg
  ## Returns: 0
  ##
  proc raw:465 {from key arg} {
    global server
    variable NamespaceCurrent
    variable configData
    variable serverdata
    variable klinedata

    if {[set index [matchServer $serverdata $server]] >= 0} then {
      set serv [lindex $serverdata $index]
    } else {
      set serv $from
    }
    if {$configData(expiretime)} then {
      set expire [convertSeconds from minutes $configData(expiretime)]
    } else {
      set expire 0
    }
    if {![string compare "" [set reason [FixReason $arg]]]} then {
      set reason "no reason"
    }
    if {[matchServer $klinedata $serv] < 0} then {
      lappend klinedata [list $serv [clock seconds] $expire $reason]
      wpLog s * $NamespaceCurrent "Error: k-lined on server `$serv': `$reason', removing from active server list and jumping..."
    } else {
      wpLog s * $NamespaceCurrent "Error: k-lined on server `$serv': `$reason', jumping..."
    }
    jump
    return 0
  }

  ##
  ## Detect and flag servers that are going to enforce a k-line for the bot
  ##
  ## 466    ERR_YOUWILLBEBANNED
  ##
  ## Args:    from, keyword, arg
  ## Returns: 0
  ##
  proc raw:466 {from key arg} {
    global server
    variable NamespaceCurrent
    variable configData
    variable serverdata
    variable klinedata

    if {[set index [matchServer $serverdata $server]] >= 0} then {
      set serv [lindex $serverdata $index]
    } else {
      set serv $from
    }
    if {$configData(expiretime)} then {
      set expire [convertSeconds from minutes $configData(expiretime)]
    } else {
      set expire 0
    }
    if {![string compare "" [set reason [FixReason $arg]]]} then {
      set reason "no reason"
    }
    if {[matchServer $klinedata $serv] < 0} then {
      lappend klinedata [list $serv [clock seconds] $expire $reason]
      wpLog s * $NamespaceCurrent "Error: server `$serv' said it's about to enforce a k-line: `$reason', removing from active server list and jumping..."
    } else {
      wpLog s * $NamespaceCurrent "Error: server `$serv' said it's about to enforce a k-line: `$reason', jumping..."
    }
    jump
    return 0
  }

  ##
  ## Detect a restricted connection and jump to the next server
  ##
  ## 484    ERR_RESTRICTED
  ##        ":Your connection is restricted!"
  ##
  ## Args:    from, keyword, arg
  ## Returns: 0
  ##
  proc raw:484 {from key arg} {
    global server
    variable NamespaceCurrent

    if {[string compare "" $server]} then {
      wpLog s * $NamespaceCurrent "Error: Connection restricted on server `$server', jumping..."
    } else {
      wpLog s * $NamespaceCurrent "Error: Connection restricted on server `$from', jumping..."
    }
    jump
    return 0
  }

  ##
  ## Check for expired k-lines
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:minutely {minute hour day month year} {
    variable klinedata

    if {[string compare "" $klinedata]} then {
      set time [clock seconds]
      set index 0
      foreach list $klinedata {
        if {[isNumber [set expire [lindex $list 2]]]} then {
          if {[expr $time - [lindex $list 1]] >= $expire} then {
            set klinedata [lreplace $klinedata $index $index]
          }
        }
        incr index
      }
    }
    return 0
  }

  ##
  ## Remove leading text from a k-line reason
  ##
  ## Args:    text
  ## Returns: k-line reason
  ##
  proc FixReason {text} {
    set reason [string range $text [expr [string first : $text] + 1] end]
    if {![string compare "you are banned from this server" \
          [string tolower [string range $reason 0 30]]]} then {
      set reason [string range $reason 31 end]
    }
    return [string trimleft $reason "- "]
  }

  ##
  ## Update server list when serverdata or klinedata is changed
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc UpdateServers {what} {
    global servers
    variable NamespaceCurrent
    variable serverdata
    variable klinedata
    variable LastUpdate

    switch -exact -- $what {
      servers {
        # FIXME: $servers suddenly changed to a shorter list will error
        if {([string compare $LastUpdate serverdata]) && \
            ([string compare $LastUpdate klinedata])} then {
          set LastUpdate servers
          set servertmp $serverdata
          # add servers from $servers to $serverdata
          set count 0
          foreach server $servers {
            set index [matchServer $servertmp $server]
            if {$index < 0} then {
              set servertmp [linsert $servertmp $count $server]
            }
            incr count
          }
          # remove servers not in $servers from $serverdata if not in $klinedata
          set count 0
          foreach server $servertmp {
            set index [matchServer $servers $server]
            if {($index < 0) && ([matchServer $klinedata $server] < 0)} then {
              set servertmp [lreplace $servertmp $count $count]
            }
            incr count
          }
          set serverdata $servertmp
          utimer 1 "set ${NamespaceCurrent}::LastUpdate \"\""
        }
      }
      serverdata {
        if {[string compare $LastUpdate servers]} then {
          set LastUpdate serverdata
          if {[string compare "" $serverdata]} then {
            set servertmp ""
            # add servers from $serverdata to $servers if not in $klinedata
            foreach server $serverdata {
              if {[matchServer $klinedata $server] < 0} then {
                lappend servertmp $server
              }
            }
            set servers $servertmp
          }
          set servertmp $klinedata
          # remove servers not listed in $serverdata from $klinedata
          set count 0
          foreach list $klinedata {
            if {[matchServer $serverdata [lindex $list 0]] == -1} then {
              set servertmp [lreplace $klinedata $count $count]
            }
            incr count
          }
          set klinedata $servertmp
          utimer 1 "set ${NamespaceCurrent}::LastUpdate \"\""
        }
      }
      klinedata {
        if {[string compare $LastUpdate serverdata]} then {
          set LastUpdate klinedata
          set servertmp $servers
          # remove servers listed in $klinedata from $servers
          foreach list $klinedata {
            set index [matchServer $servertmp [lindex $list 0]]
            if {$index >= 0} then {
              set servertmp [lreplace $servertmp $index $index]
            }
          }
          # add servers from $serverdata to $servers if not in $klinedata
          set count 0
          foreach server $serverdata {
            set index [matchServer $servertmp $server]
            if {($index < 0) && ([matchServer $klinedata $server] < 0)} then {
              set servertmp [linsert $servertmp $count $server]
            }
            incr count
          }
          set servers $servertmp
          utimer 1 "set ${NamespaceCurrent}::LastUpdate \"\""
        }
      }
    }
    return
  }

  ##
  ## Trace server and kline data variable changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceData {name1 name2 op} {
    switch -exact -- $name1 {
      servers {
        UpdateServers servers
      }
      serverdata {
        variable serverdataChanged
        UpdateServers serverdata
        set serverdataChanged 1
      }
      klinedata {
        variable klinedataChanged
        UpdateServers klinedata
        set klinedataChanged 1
      }
    }
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    global servers
    variable NamespaceCurrent
    variable serverdata
    variable klinedata

    trace $action servers w ${NamespaceCurrent}::TraceData
    trace $action serverdata w ${NamespaceCurrent}::TraceData
    trace $action klinedata w ${NamespaceCurrent}::TraceData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    global servers
    variable NamespaceCurrent
    variable serverdata
    variable klinedata
    variable serverdataChanged
    variable OldServers
    variable LastUpdate

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        if {![info exists OldServers]} then {
          set OldServers $servers
        }
        set tmpservers [join $servers]
        regsub -all -- "  " $tmpservers " " tmpservers
        # Workaround for eggdrop's traced servers variable
        catch {
          unset servers
        }
        set servers $tmpservers
        if {(![info exists serverdata]) || (![string compare "" $serverdata])} then {
          set serverdata $servers
          set serverdataChanged 1
        }
        if {![info exists klinedata]} then {
          set klinedata ""
        }
        if {![info exists LastUpdate]} then {
          set LastUpdate ""
        }
        set LastUpdate ""
        UpdateServers serverdata
        UpdateServers klinedata
      }
      unload {
        if {[info exists OldServers]} then {
          set servers $OldServers
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "dcc raw time"
    }
    foreach type $what {
      switch -exact -- $type {
        dcc {
          $action dcc m|- servers ${NamespaceCurrent}::dcc:servers
          $action dcc m|- +server ${NamespaceCurrent}::dcc:+server
          $action dcc m|- -server ${NamespaceCurrent}::dcc:-server
          $action dcc m|- mvserver ${NamespaceCurrent}::dcc:mvserver
          $action dcc m|- klines ${NamespaceCurrent}::dcc:klines
          $action dcc m|- +kline ${NamespaceCurrent}::dcc:+kline
          $action dcc m|- -kline ${NamespaceCurrent}::dcc:-kline
          $action dcc m|- jump ${NamespaceCurrent}::dcc:jump
        }
        raw {
          $action raw -|- 436 ${NamespaceCurrent}::raw:436
          $action raw -|- 465 ${NamespaceCurrent}::raw:465
          $action raw -|- 466 ${NamespaceCurrent}::raw:466
          $action raw -|- 484 ${NamespaceCurrent}::raw:484
        }
        time {
          $action time -|- "* * * * *" ${NamespaceCurrent}::time:minutely
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    expiretime 21600
  }

  # Configuration data descriptions
  array set configDataDesc {
    expiretime "\[minutes\] default expire time for k-lines"
  }

  # Set datafile information
  array set dataOptions {
    serverdata "list servman-server.data server"
    klinedata "list servman-kline.data k-line"
  }

} ;# namespace servman

return
