###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: skel.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: skel
# version: 1.1.0.0
# config: skel.conf
# description: Silly example module that adds a dcc, msg, and public command \
#	called `woof' that displays one of 2 random lines each time it's used.
# requires: commandChar put validHandle authCheck

namespace eval skel {

  ##
  ## Silly example command: pub
  ##
  ## Args:    nick, uhost, hand, chan, arg
  ## Returns: 1
  ##
  proc pub:woof {nick uhost hand chan arg} {
    return [woof pub putmsg $chan putmsg $chan $nick $uhost $hand $chan $arg]
  }

  ##
  ## Silly example command: msg
  ##
  ## Args:    nick, uhost, hand, arg
  ## Returns: 1
  ##
  proc msg:woof {nick uhost hand arg} {
    return [woof msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Silly example command: dcc
  ##
  ## Args:    hand, idx, arg
  ## Returns: 1
  ##
  proc dcc:woof {hand idx arg} {
    return [woof dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  ##
  ## Silly example command
  ##
  ## Args:    type {pub/msg/dcc}, resp1 {putnot/putmsg/putdcc},
  ##          dest1 {nick/idx}, resp2 {putnot/putmsg/putdcc},
  ##          dest2 {nick/idx}, nick, uhost, hand, chan, arg
  ## Returns: 1 if the user is auth'd
  ##          0 otherwise
  ##
  proc woof {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable configData

    if {([validHandle $hand]) && \
        ([authCheck $type woof $nick $uhost $hand $chan])} then {
      if {[expr [rand 2] - 1]} then {
        put $resp1 $dest1 $configData(text1)
      } else {
        put $resp1 $dest1 $configData(text2)
      }
      return 1
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc"
    }
    foreach type $what {
      switch -exact -- $type {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub p|- ${char}woof ${NamespaceCurrent}::pub:woof
        }
        msg {
          $action msg p|- woof ${NamespaceCurrent}::msg:woof
        }
        dcc {
          $action dcc -|- woof ${NamespaceCurrent}::dcc:woof
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    text1 "Woof! Woof!"
    text2 "Arrroooooo!"
  }

  # Configuration data descriptions
  array set configDataDesc {
    text1 "\[text\] some example text"
    text2 "\[text\] more example text"
  }

} ;# namespace skel

return
