###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: userping.tcl,v 1.6 2001/06/07 08:19:26 tothwolf Exp $
##

# name: userping
# version: 1.3.0.0
# config: userping.conf
# description: Adds a public channel command called `ping' that allows the \
#	bot to ping users and display the lagtime.
# requires: isbotnick duration commandChar putctcp putmsg putnot isNumber \
#	validHandle calibrateClicksPerSecond clicksPerSecond authCheck \
#	floodCheck

namespace eval userping {

  ##
  ## Pings the user who sent the command, or another user in the channel
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if the user is auth'd
  ##          0 otherwise
  ##
  # FIXME: add msg and dcc commands
  proc pub:ping {nick uhost hand chan arg} {
    variable configData
    variable Active
    variable Users

    if {([floodCheck $nick $uhost login]) && \
        (![isbotnick $nick]) && \
        ((!$configData(authmode)) || \
         ([validHandle $hand]) && \
         (($configData(authmode) == 1) || \
          (($configData(authmode) == 2) && \
           ([authCheck pub ping $nick $uhost $hand $chan]))))} then {
      if {[string compare "" $arg]} then {
        set who [lindex [split [string trimright $arg]] 0]
      } else {
        set who $nick
      }
      if {[onchan $who $chan]} then {
        set Active 1
        set Users([string tolower $who]) "$chan $nick"
        calibrateClicksPerSecond
        putctcp $who "PING [clock seconds] [clock clicks]"
      } else {
        putnot $nick "$who isn't on $chan."
      }
      return 1
    }
    return 0
  }

  ##
  ## Display ping reply from a user
  ##
  ## Args:    nick, user@host, handle, dest, keyword, arg
  ## Returns: 0
  ##
  proc ctcr:ping {nick uhost hand dest key arg} {
    variable configData
    variable Active
    variable Users

    set now "[clock seconds] [clock clicks]"
    if {($Active) && ([info exists Users([set who [string tolower $nick]])])} then {
      set seconds [lindex $arg 0]
      if {[isNumber $seconds]} then {
        if {([set time [expr [lindex $now 0] - $seconds]] < 60) && \
            ([isNumber [set clicks [lindex $arg 1]]]) && \
            ([set clicktime [expr (([lindex $now 1] - $clicks) + 0.0) / [clicksPerSecond]]] >= 0)} then {
          set reply "[format "%f" $clicktime] seconds"
        } else {
          set reply [duration $time]
        }
        if {$configData(pubreply)} then {
          putmsg [lindex $Users($who) 0] "Ping reply from $nick: $reply"
        } else {
          putnot [lindex $Users($who) 1] "Ping reply from $nick: $reply"
        }
      }
    } elseif {[info exists Users($who)]} then {
      unset Users($who)
    }
    if {![string compare "" [array names Users]]} then {
      set Active 0
    }
    return 0
  }

  ##
  ## Clear invalid data from Users array once an hour
  ##
  ## Args:    minute, hour, day, month, year
  ## Returns: 0
  ##
  proc time:hourly {minute hour day month year} {
    variable Users
    variable Active

    if {[string compare "" [set users [array get Users]]]} then {
      if {$Active} then {
        foreach {name data} $users {
          if {![onchan $name [lindex $data 0]]} then {
            unset Users($name)
          }
        }
      } else {
        unset Users
      }
    } elseif {$Active} then {
      set Active 0
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent
    variable Active

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        if {![info exists Active]} then {
          set Active 0
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub ctcr time"
    }
    foreach type $what {
      switch -exact -- $type {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}ping ${NamespaceCurrent}::pub:ping
        }
        ctcr {
          $action ctcr -|- PING ${NamespaceCurrent}::ctcr:ping
        }
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    pubreply 1
    authmode 2
  }

  # Configuration data descriptions
  array set configDataDesc {
    pubreply "\[0/1\] show ping replys on the channel?"
    authmode "\[0/1/2\] 0 - allow anyone to use ping, 1 - only regonized users, 2 - only auth'd users"
  }

} ;# namespace userping

return
