###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: weather.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: weather
# version: 1.5.6.3
# config: weather.conf
# description:
# requires: this_module_needs_to_be_converted

namespace eval weather {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # set data information
  set data(weathdata) "1 $conf(weathdatafile) ${NamespaceCurrent}::weathdata"

  # load data
  module_data load "" 1 0 "1 $conf(weathdatafile) ${NamespaceCurrent}::weathdata"

  # show weather data; pub
  proc pub:weath {nick uhost hand chan arg} {
    return [weath pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  # show weather data; msg
  proc msg:weath {nick uhost hand arg} {
    return [weath msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # show weather data; dcc
  proc dcc:weath {hand idx arg} {
    return [weath dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # show weather data
  proc weath {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {([validHandle $hand]) && ([validNick $nick])} then {
      variable weathdata
      put $resp1 $dest1 "Last Update: [ctime $weathdata(time)], Temperature: $weathdata(temperature), Humidity: $weathdata(humidity), Dewpoint: $weathdata(dewpoint), Wind: $weathdata(wind), Pressure: $weathdata(pressure), Conditions: $weathdata(conditions)"
      return 1
    }
    return 0
  }

  # show temperature; pub
  proc pub:temp {nick uhost hand chan arg} {
    return [temp pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  # show temperature; msg
  proc msg:temp {nick uhost hand arg} {
    return [temp msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # show temperature; dcc
  proc dcc:temp {hand idx arg} {
    return [temp dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # show temperature
  proc temp {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {([validHandle $hand]) && ([validNick $nick])} then {
      variable weathdata
      put $resp1 $dest1 "Current temperature: $weathdata(temperature)"
      return 1
    }
    return 0
  }

  # show wind speed; pub
  proc pub:wind {nick uhost hand chan arg} {
    return [wind pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  # show wind speed; msg
  proc msg:wind {nick uhost hand arg} {
    return [wind msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # show wind speed; dcc
  proc dcc:wind {hand idx arg} {
    return [wind dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # show wind speed
  proc wind {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {([validHandle $hand]) && ([validNick $nick])} then {
      variable weathdata
      put $resp1 $dest1 "Current wind conditions: $weathdata(wind)"
      return 1
    }
    return 0
  }

  # show partial weather data on join
  proc join:weath {nick uhost hand chan} {
    if {([validHandle $hand]) && ([validNick $nick])} then {
      variable weathdata
      putnot $nick "The temperature is $weathdata(temperature), and the conditions are $weathdata(conditions)."
    }
    return 0
  }

  # get weather data; pub
  proc pub:getweath {nick uhost hand chan arg} {
    if {[auth_check pub $nick $uhost $hand $chan getweath]} then {
      putnot $nick "Retrieving weather data..."
      getweath
      return 1
    }
    return 0
  }

  # get weather data; msg
  proc msg:getweath {nick uhost hand arg} {
    if {[auth_check msg $nick $uhost $hand "" getweath]} then {
      putnot $nick "Retrieving weather data..."
      getweath
      return 1
    }
    return 0
  }

  # get weather data; dcc
  proc dcc:getweath {hand idx arg} {
    putdcc $idx "Retrieving weather data..."
    getweath
    return 1
  }

  # get weather data
  proc getweath {} {
    variable conf
    variable weathdata
    variable weathcount
    variable NamespaceParent
    variable NamespaceCurrent
    if {[info exists weathdata(time)]} then {
      set weathdata(time_last) $weathdata(time)
    } else {
      set weathdata(time_last) [clock seconds]
    }
    set weathdata(time) [clock seconds]
    set weathcount(current) 0
    set weathcount(next) 0
    set weathcount(item) none
    if {$conf(debug)} then {
      set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] w]
      close $fd
    }
    if {[catch {set idx [connect $conf(host) $conf(port)]} result]} then {
      wpLog o * $NamespaceCurrent "Error: unable to connect to `$conf(host):$conf(port)': $result"
    } else {
      control $idx control_getweath
      putdcc $idx "GET $conf(path) HTTP/1.0"
      putdcc $idx "Pragma: no-cache"
      putdcc $idx "From: [contact_address]"
      putdcc $idx "User-Agent: wolfpack.tcl\[weather.tcl\]/ver"
      putdcc $idx ""
      return 1
    }
    return 0
  }

  # grab the data we want
  proc control_getweath {idx text} {
    variable conf
    variable weathdata
    variable weathdata_changed
    variable weathcount
    NamespaceParent
    set textlower [string tolower $text]
    set strip "</tr> <td> </td> <b> </b> &#176;"
    incr weathcount(current)
    if {([string compare none $weathcount(item)]) && \
        ($weathcount(current) == $weathcount(next))} then {
      if {[info exists weathdata($weathcount(item))]} then {
        set weathdata($weathcount(item)_last) $weathdata($weathcount(item))
      } else {
        set weathdata($weathcount(item)_last) none
      }
      set weathdata($weathcount(item)) [replaceExpr [replaceExpr $text $strip] "  " " "]
      set weathdata_changed 1
    }
    if {[regexp ".*<td>temperature</td>.*" $textlower]} then {
      set weathcount(item) temperature
      set weathcount(next) [expr $weathcount(current) + 1]
    } elseif {[regexp ".*<td>humidity</td>.*" $textlower]} then {
      set weathcount(item) humidity
      set weathcount(next) [expr $weathcount(current) + 1]
    } elseif {[regexp ".*<td>dewpoint</td>.*" $textlower]} then {
      set weathcount(item) dewpoint
      set weathcount(next) [expr $weathcount(current) + 1]
    } elseif {[regexp ".*<td>wind</td>.*" $textlower]} then {
      set weathcount(item) wind
      set weathcount(next) [expr $weathcount(current) + 1]
    } elseif {[regexp ".*<td>pressure</td>.*" $textlower]} then {
      set weathcount(item) pressure
      set weathcount(next) [expr $weathcount(current) + 1]
    } elseif {[regexp ".*<td>conditions</td>.*" $textlower]} then {
      set weathcount(item) conditions
      set weathcount(next) [expr $weathcount(current) + 1]
    }
    if {$conf(debug)} then {
      set fd [open [file join [set ${NamespaceParent}::conf(datapath)] $conf(debugfile)] a]
      puts $fd $text
      close $fd
    }
  }

  # show if the weather has changed
  proc show_changes {chan} {
    variable weathdata
    set reply ""
    if {[string compare $weathdata(conditions_last) $weathdata(conditions)]} then {
      lappend reply "Conditions have changed to: $weathdata(conditions),"
    }
    if {[string compare $weathdata(temperature_last) $weathdata(temperature)]} then {
      lappend reply "Temperature has changed to: $weathdata(temperature),"
    }
    if {[string compare $weathdata(wind_last) $weathdata(wind)]} then {
      lappend reply "Wind has changed to: $weathdata(wind),"
    }
    if {[string compare "" $reply]} then {
      putmsg $chan [string trimright [join $reply] ,]
    }
  }

  # get current weather data every 15 minutes
  proc time:min_15 {min hour day month year} {
    variable NamespaceCurrent
    variable conf
    get_weathdata
    if {[validchan $conf(channel)]} then {
      utimer 30 "${NamespaceCurrent}::show_changes $conf(channel)"
    }
    return 0
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc join time"
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}weath ${NamespaceCurrent}::pub:weath
          $action pub -|- ${char}temp ${NamespaceCurrent}::pub:temp
          $action pub -|- ${char}wind ${NamespaceCurrent}::pub:wind
          $action pub m|- ${char}getweath ${NamespaceCurrent}::pub:getweath
        }
        msg {
          $action msg -|- weath ${NamespaceCurrent}::msg:weath
          $action msg -|- temp ${NamespaceCurrent}::msg:temp
          $action msg -|- wind ${NamespaceCurrent}::msg:wind
          $action msg m|- getweath ${NamespaceCurrent}::msg:getweath
        }
        dcc {
          $action dcc -|- weath ${NamespaceCurrent}::dcc:weath
          $action dcc -|- temp ${NamespaceCurrent}::dcc:temp
          $action dcc -|- wind ${NamespaceCurrent}::dcc:wind
          $action dcc m|- getweath ${NamespaceCurrent}::dcc:getweath
        }
        join {
          $action join -|- * ${NamespaceCurrent}::join:weath
        }
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:min_15
          $action time -|- "15 * * * *" ${NamespaceCurrent}::time:min_15
          $action time -|- "30 * * * *" ${NamespaceCurrent}::time:min_15
          $action time -|- "45 * * * *" ${NamespaceCurrent}::time:min_15
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    channel #nochannel
    debug 0
    debugfile weather.debug
    host www.wunderground.com
    path /global/stations/71486.html
    port 80
    weathdatafile weather-cache.data
  }

  # Configuration data descriptions
  array set configDataDesc {
    channel "\[#channel\] channel name that will get weather updates"
    debug "\[0/1\] write to a debug file (this WILL slow things down)"
    debugfile "\[file\] file to write raw html to"
    host "\[host\] http server host"
    path "\[url\] url for data"
    port "\[port\] http server port"
    weathdatafile "\[file\] file that weather data is stored in"
  }

} ;# namespace weather

return
