###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: pointdata.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpg-pointdata
# version: 1.0.0.0
# config: wrpg-pointdata.conf
# description: Wolf RPG user hit point and food point data support.
# requires: isNumber convertTime clearFields whoisFields convertSeconds \
#	userStatusData validHandle
# provides: userPointData updatePointData

namespace eval wrpg-pointdata {

  ##
  ## Handle a user's hp/fp data
  ##
  ## Args:    user, what {hp|fp}, action {incr|decr|set|list|itime|dtime|
  ##          check|update|test1|test2|limit|clear}, amount
  ## Returns: hp/fp, time, 0 or -1
  ##
  proc userPointData {user action what args} {
    variable configData

    set amount [lindex $args 0]
    set uwhat [string toupper $what]
    switch -exact -- $action {
      incr {
        set time [clock seconds]
        set data [getuser $user XTRA $configData(${what}field)]
        set olddata $data
        incr data $amount
        if {($data >= 0) && ($data <= $configData(${what}limit))} then {
          setuser $user XTRA [string toupper $configData(${what}field)] $data
          setuser $user XTRA [string toupper $configData(${what}itimefield)] $time
          setuser $user XTRA [string toupper $configData(${what}iinfofield)] "Last $uwhat increase: [clock format $time -format "%H:%M"]"
          return $data
        }
        return $olddata
      }
      decr {
        set time [clock seconds]
        set data [getuser $user XTRA $configData(${what}field)]
        set olddata $data
        incr data -$amount
        if {($data >= 0) && ($data <= $configData(${what}limit))} then {
          setuser $user XTRA [string toupper $configData(${what}field)] $data
          setuser $user XTRA [string toupper $configData(${what}dtimefield)] $time
          setuser $user XTRA [string toupper $configData(${what}dinfofield)] "Last $uwhat decrease: [clock format $time -format "%H:%M"]"
          return $data
        }
        return $olddata
      }
      set {
        if {($amount >= 0) && ($amount <= $configData(${what}limit))} then {
          setuser $user XTRA [string toupper $configData(${what}field)] $amount
          return $amount
        }
        return [getuser $user XTRA $configData(${what}field)]
      }
      list {
        return [getuser $user XTRA $configData(${what}field)]
      }
      itime {
        return [getuser $user XTRA $configData(${what}itimefield)]
      }
      dtime {
        return [getuser $user XTRA $configData(${what}dtimefield)]
      }
      check {
        set time [clock seconds]
        if {(![string length \
               [set data [getuser $user XTRA $configData(${what}field)]]]) || \
            ($data > $configData(${what}limit))} then {
          set data $configData(${what}limit)
          setuser $user XTRA [string toupper $configData(${what}field)] $data
        }
        if {(![string length \
               [set itime \
                [getuser $user XTRA $configData(${what}itimefield)]]]) || \
            ($itime > $time)} then {
          setuser $user XTRA [string toupper $configData(${what}itimefield)] $time
          set ireset 1
        }
        if {(![string length \
               [getuser $user XTRA $configData(${what}iinfofield)]]) || \
            ([info exists ireset])} then {
          setuser $user XTRA [string toupper $configData(${what}iinfofield)] "Last $uwhat increase: never"
        }
        if {(![string length \
               [set dtime \
                [getuser $user XTRA $configData(${what}dtimefield)]]]) || \
            ($dtime > $time)} then {
          setuser $user XTRA [string toupper $configData(${what}dtimefield)] $time
          set dreset 1
        }
        if {(![string length \
               [getuser $user XTRA $configData(${what}dinfofield)]]) || \
            ([info exists dreset])} then {
          setuser $user XTRA [string toupper $configData(${what}dinfofield)] "Last $uwhat decrease: never"
        }
        return $data
      }
      update {
        if {[isNumber \
             [set itime \
              [getuser $user XTRA $configData(${what}itimefield)]]]} then {
          setuser $user XTRA [string toupper $configData(${what}iinfofield)] [convertTime $itime]
        } else {
          setuser $user XTRA [string toupper $configData(${what}iinfofield)] "Last $uwhat increase: never"
        }
        if {[isNumber \
             [set dtime \
              [getuser $user XTRA $configData(${what}dtimefield)]]]} then {
          setuser $user XTRA [string toupper $configData(${what}dinfofield)] [convertTime $dtime]
        } else {
          setuser $user XTRA [string toupper $configData(${what}dinfofield)] "Last $uwhat decrease: never"
        }
        return 0
      }
      test1 {
        if {[getuser $user XTRA $configData(${what}field)] <= $configData(${what}ok)} then {
          return 0
        }
        return 1
      }
      test2 {
        if {[getuser $user XTRA $configData(${what}field)] >= $configData(${what}limit)} then {
          return 0
        }
        return 1
      }
      limit {
        return $configData(${what}limit)
      }
      clear {
        clearFields $user "$configData(${what}field) $configData(${what}itimefield) $configData(${what}iinfofield) $configData(${what}dtimefield) $configData(${what}dinfofield)"
        return 0
      }
    }
    return -1
  }

  ##
  ## Update hp anf fp counts
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc updatePointData {} {
    variable configData

    set time [clock seconds]
    if {[string compare "" \
         [set users [userStatusData "" listusers status pack]]]} then {
      foreach user $users {
        if {![isNumber [set hp [userPointData $user list hp]]]} then {
          set hp [userPointData $user check hp]
        }
        if {![isNumber [set fp [userPointData $user list fp]]]} then {
          set fp [userPointData $user check fp]
        }
        if {(!$fp) && ($hp > 0)} then {
          if {![info exists hpdseconds]} then {
            set hpdseconds [convertSeconds from hours $configData(hpdecrtime)]
          }
          if {[expr $time - [userPointData $user dtime hp]] >= $hpdseconds} then {
            userPointData $user decr hp 1
          }
        } elseif {$fp > 0} then {
          if {![info exists fpdseconds]} then {
            set fpdseconds [convertSeconds from hours $configData(fpdecrtime)]
          }
          if {[expr $time - [userPointData $user dtime fp]] >= $fpdseconds} then {
            userPointData $user decr fp 1
          }
        }
        if {($hp >= 0) && ($hp < $configData(hplimit)) && \
            ($fp >= $configData(fpok))} then {
          if {![info exists hpiseconds]} then {
            set hpiseconds [convertSeconds from hours $configData(hpincrtime)]
          }
          if {[expr $time - [userPointData $user itime hp]] >= $hpiseconds} then {
            userPointData $user incr hp 1
          }
        }
      }
    }
    return
  }

  ##
  ## Trace configuration changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceConfigData {name1 name2 op} {
    variable configData
    variable oldConfigData

    if {[string compare $oldConfigData($name2) $configData($name2)]} then {
      switch -exact -- $name2 {
        fpdinfofield -
        fpdtimefield -
        fpfield -
        fpiinfofield -
        fpitimefield -
        hpdinfofield -
        hpdtimefield -
        hpfield -
        hpiinfofield -
        hpitimefield {
          moveFields $oldConfigData($name2) [string toupper $configData($name2)]
          whoisFields update $configData($name2) $oldConfigData($name2)
        }
      }
      set oldConfigData($name2) $configData($name2)
    }
    return
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    variable NamespaceCurrent
    variable configData

    trace $action configData w ${NamespaceCurrent}::TraceConfigData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable configData
    variable oldConfigData
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        foreach {name data} [array get configData] {
          if {![info exists oldConfigData($name)]} then {
            set oldConfigData($name) $data
          }
        }
        whoisFields update "$configData(hpfield) $configData(hpiinfofield) $configData(hpdinfofield) $configData(fpfield) $configData(fpiinfofield) $configData(fpdinfofield)" \
                           "$oldConfigData(hpfield) $oldConfigData(hpiinfofield) $oldConfigData(hpdinfofield) $oldConfigData(fpfield) $oldConfigData(fpiinfofield) $oldConfigData(fpdinfofield)"
      }
      unload {
        whoisFields remove "$configData(hpfield) $configData(hpiinfofield) $configData(hpdinfofield) $configData(fpfield) $configData(fpiinfofield) $configData(fpdinfofield)"
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    fpdecrtime 24
    fpdinfofield fpdinfo
    fpdtimefield fpdtime
    fpfield fp
    fpiinfofield fpiinfo
    fpitimefield fpitime
    fplimit 10
    fpok 5
    hpdecrtime 24
    hpdinfofield hpdinfo
    hpdtimefield hpdtime
    hpfield hp
    hpiinfofield hpiinfo
    hpincrtime 48
    hpitimefield hpitime
    hplimit 10
    hpok 5
  }

  # Configuration data descriptions
  array set configDataDesc {
    fpdecrtime "\[hours\] time between fp drops for a user"
    fpdinfofield "\[field\] field for user's last fp decrease info"
    fpdtimefield "\[field\] field for user's last fp decrease time"
    fpfield "\[field\] field for user's fp"
    fpiinfofield "\[field\] field for user's last fp increase info"
    fpitimefield "\[field\] field for user's last fp increase time"
    fplimit "\[number\] number of foodpoints a user has"
    fpok "\[number\] number that a user's fp must reach before it registers as ok"
    hpdecrtime "\[hours\] time between hp drops for a user if fp is 0"
    hpdinfofield "\[field\] field for user's last hp decrease info"
    hpdtimefield "\[field\] field for user's last hp decrease time"
    hpfield "\[field\] field for user's hp"
    hpiinfofield "\[field\] field for user's last hp increase info"
    hpincrtime "\[hours\] hours before a user's hp is increased"
    hpitimefield "\[field\] field for user's last hp increase time"
    hplimit "\[number\] number of hitpoints a user has"
    hpok "\[number\] number that a user's hp must reach before it registers as ok"
  }

} ;# namespace wrpg-pointdata

return
