###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: statuscmds.tcl,v 1.5 2001/06/06 09:36:14 tothwolf Exp $
##

# name: wrpg-statuscmds
# version: 1.0.0.0
# description: Wolf RPG user status dcc, msg and public commands
# requires: authCheck commandChar findUser checkUser putList splitList put \
#	validHandle duration userPointData userStatusData \
#	userAssessmentData floodCheck

namespace eval wrpg-statuscmds {

  ##
  ## Add pack status for a user: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:addpack {nick uhost hand chan arg} {
    return [addpack pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Add pack status for a user: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:addpack {nick uhost hand arg} {
    return [addpack msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Add pack status for a user: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:addpack {hand idx arg} {
    return [addpack dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Add pack status for a user
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc addpack {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type addpack $nick $uhost $hand $chan]} then {
      set arg [split [string trimright $arg]]
      set who [lindex $arg 0]
      set pos [join [lrange $arg 1 end]]
      if {![string length $who]} then {
        put $resp2 $dest2 "Usage: [commandChar $type]addpack <nick|hand> \[position\]"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots can't be pack."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "$user is already pack."
            }
            assessment {
              if {[userStatusData $user set status pack]} then {
                userAssessmentData $user del
                if {[string length $pos]} then {
                  userStatusData $user set pos $pos
                } else {
                  userStatusData $user check pos
                }
                userStatusData $user check level
                userPointData $user check hp
                userPointData $user check fp
                put $resp1 $dest1 "Updated status for $user from assessment to pack."
              } else {
                put $resp1 $dest1 "Error updating status for $user from assessment to pack."
              }
            }
            default {
              if {[userStatusData $user set status pack]} then {
                if {[string length $pos]} then {
                  userStatusData $user set pos $pos
                } else {
                  userStatusData $user check pos
                }
                userStatusData $user check level
                userPointData $user check hp
                userPointData $user check fp
                put $resp1 $dest1 "Added pack status for $user."
              } else {
                put $resp1 $dest1 "Error adding pack status for $user."
              }
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Remove pack status from a user: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:delpack {nick uhost hand chan arg} {
    return [delpack pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Remove pack status from a user: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:delpack {nick uhost hand arg} {
    return [delpack msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Remove pack status from a user: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:delpack {hand idx arg} {
    return [delpack dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Remove pack status from a user
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc delpack {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type delpack $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {![string length $who]} then {
        put $resp2 $dest2 "Usage: [commandChar $type]delpack <nick|hand>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots can't be pack."
            }
          }
        } elseif {![userStatusData $user compare status pack]} then {
          put $resp1 $dest1 "$user is not pack."
        } elseif {[userStatusData $user clear status]} then {
          userStatusData $user clear pos
          userStatusData $user clear level
          userPointData $user clear hp
          userPointData $user clear fp
          put $resp1 $dest1 "Removed pack status from $user."
        } else {
          put $resp1 $dest1 "Error removing pack status for $user."
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Display pack info for a user: pub
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listpack {nick uhost hand chan arg} {
    return [listpack pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Display pack info for a user: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listpack {nick uhost hand arg} {
    return [listpack msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Display pack info for a user: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listpack {hand idx arg} {
    return [listpack dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Display pack info for a user
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc listpack {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type listpack $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {[string length $who]} then {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have pack info."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "Info for $user: HP \[[userPointData $user list hp]\]; FP \[[userPointData $user list fp]\]; Level \[[userStatusData $user list level]\]; Position \[[userStatusData $user list pos]\]"
            }
            assessment {
              put $resp1 $dest1 "$user is under assessment."
            }
            default {
              put $resp1 $dest1 "$user is not pack."
            }
          }
        }
      } else {
        if {[string compare "" [set users [userStatusData "" listusers status pack]]]} then {
          set count 0
          foreach user $users {
            append list "$user: HP \[[userPointData $user list hp]\]; FP \[[userPointData $user list fp]\]; Level \[[userStatusData $user list level]\]; Position \[[userStatusData $user list pos]\], "
            incr count
          }
          putList $resp1 $dest1 [splitList $list 160 , ", "] "I know the following pack members ($count): " "(cont): "
        } else {
          put $resp1 $dest1 "No users are pack."
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Add assessment status for a user: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:addas {nick uhost hand chan arg} {
    return [addas pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Add assessment status for a user: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:addas {nick uhost hand arg} {
    return [addas msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Add assessment status for a user: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:addas {hand idx arg} {
    return [addas dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Add assessment status for a user
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc addas {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type addas $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {![string length $who]} then {
        put $resp2 $dest2 "Usage: [commandChar $type]addas <nick|hand>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots can't be under assessment."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "$user is already pack."
            }
            assessment {
              put $resp1 $dest1 "$user is already under assessment."
            }
            default {
              if {[userStatusData $user set status assessment]} then {
                userAssessmentData $user add
                put $resp1 $dest1 "Added assessment status for $user."
              } else {
                put $resp1 $dest1 "Error adding assessment status for $user."
              }
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Remove assessment status from a user: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:delas {nick uhost hand chan arg} {
    return [delas pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Remove assessment status from a user: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:delas {nick uhost hand arg} {
    return [delas msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Remove assessment status from a user: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:delas {hand idx arg} {
    return [delas dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Remove assessment status from a user
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc delas {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type delas $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {![string length $who]} then {
        put $resp2 $dest2 "Usage: [commandChar $type]delas <nick|hand>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots can't be under assessment."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "$user is already pack."
            }
            assessment {
              if {[userStatusData $user clear status]} then {
                userAssessmentData $user del
                put $resp1 $dest1 "Removed assessment status for $user."
              } else {
                put $resp1 $dest1 "Error removing assessment status for $user."
              }
            }
            default {
              put $resp1 $dest1 "$user is not under assessment."
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Display assessment info for a user: pub
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listas {nick uhost hand chan arg} {
    return [listas pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Display assessment info for a user: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listas {nick uhost hand arg} {
    return [listas msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Display assessment info for a user: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listas {hand idx arg} {
    return [listas dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Display assessment info for a user
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc listas {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type listpack $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {[string length $who]} then {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have assessment info."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "$user is pack."
            }
            assessment {
              switch -exact -- [set timeleft [userAssessmentData $user timeleft]] {
                -1 {
                  put $resp1 $dest1 "Assessment started for $user: [userAssessmentData $user started]"
                }
                0 {
                  put $resp1 $dest1 "Assessment over for $user."
                }
                default {
                  put $resp1 $dest1 "Assessment time left for $user: [duration $timeleft]"
                }
              }
            }
            default {
              put $resp1 $dest1 "$user is not under assessment."
            }
          }
        }
      } else {
        if {[string compare "" [set users [userStatusData "" listusers status assessment]]]} then {
          set count 0
          foreach user $users {
            switch -exact -- [set timeleft [userAssessmentData $user timeleft]] {
              -1 {
                append list "$user: started [clock format [userAssessmentData $user time] -format "%a %b %d %Y"], "
              }
              0 {
                append list "$user: assessment over, "
              }
              default {
                append list "$user: [duration $timeleft] left, "
              }
            }
            incr count
          }
          putList $resp1 $dest1 [splitList $list 160 , ", "] "Users under assessment ($count): " "(cont): "
        } else {
          put $resp1 $dest1 "No users are under assessment."
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Change a user's position: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:chpos {nick uhost hand chan arg} {
    return [chpos pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Change a user's position: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:chpos {nick uhost hand arg} {
    return [chpos msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Change a user's position: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:chpos {hand idx arg} {
    return [chpos dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Change a user's position
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc chpos {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type chpos $nick $uhost $hand $chan]} then {
      set arg [split [string trimright $arg]]
      set who [lindex $arg 0]
      set pos [join [lrange $arg 1 end]]
      if {(![string length $who]) || (![string length $pos])} then {
        put $resp2 $dest2 "Usage: [commandChar $type]chpos <nick|hand> <position>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have a pack position."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              if {[string compare none [string tolower $pos]]} then {
                if {[userStatusData $user set pos $pos]} then {
                  put $resp1 $dest1 "Set position for $user to: $pos"
                } else {
                  put $resp1 $dest1 "Error setting position for $user"
                }
              } elseif {[userStatusData $user clear pos]} then {
                put $resp1 $dest1 "Cleared position for $user."
              } else {
                put $resp1 $dest1 "Error clearing position for $user"
              }
            }
            assessment {
              put $resp1 $dest1 "$user is under assessment."
            }
            default {
              put $resp1 $dest1 "$user is not pack."
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## List a user's position: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listpos {nick uhost hand chan arg} {
    return [listpos pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## List a user's position: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listpos {nick uhost hand arg} {
    return [listpos msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## List a user's position: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listpos {hand idx arg} {
    return [listpos dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## List a user's position
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc listpos {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type listpos $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {![string length $who]} then {
        put $resp2 $dest2 "Usage: [commandChar $type]listpos <nick|hand>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have a pack position."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "Position for $user: [userStatusData $user list pos]"
            }
            assessment {
              put $resp1 $dest1 "$user is under assessment."
            }
            default {
              put $resp1 $dest1 "$user is not pack."
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Change a user's level: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:chlev {nick uhost hand chan arg} {
    return [chlev pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## Change a user's level: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:chlev {nick uhost hand arg} {
    return [chlev msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## Change a user's level: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:chlev {hand idx arg} {
    return [chlev dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## Change a user's level
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc chlev {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type chlev $nick $uhost $hand $chan]} then {
      set arg [split [string trimright $arg]]
      set who [lindex $arg 0]
      set level [lindex $arg 1]
      if {(![string length $who]) || (![string length $level])} then {
        put $resp2 $dest2 "Usage: [commandChar $type]chlev <nick|hand> <level>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have a pack level."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              if {[userStatusData $user set level $level]} then {
                put $resp1 $dest1 "Set level for $user to: $level"
              } else {
                put $resp1 $dest1 "Error setting level for $user"
              }
            }
            assessment {
              put $resp1 $dest1 "$user is under assessment."
            }
            default {
              put $resp1 $dest1 "$user is not pack."
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## List a user's level: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listlev {nick uhost hand chan arg} {
    return [listlev pub putnot $nick putnot $nick $nick $uhost $hand \
            $chan $arg]
  }

  ##
  ## List a user's level: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listlev {nick uhost hand arg} {
    return [listlev msg putnot $nick putnot $nick $nick $uhost $hand \
            "" $arg]
  }

  ##
  ## List a user's level: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listlev {hand idx arg} {
    return [listlev dcc putdcc $idx putdcc $idx "" "" $hand \
            [lindex [console $idx] 0] $arg]
  }

  ##
  ## List a user's level
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc listlev {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[authCheck $type listlev $nick $uhost $hand $chan]} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {![string length $who]} then {
        put $resp2 $dest2 "Usage: [commandChar $type]listlev <nick|hand>"
      } else {
        set user [findUser $who $chan]
        if {[set result [checkUser $user]] != 1} then {
          switch -exact -- $result {
            0 {
              put $resp1 $dest1 "I don't know who $who is."
            }
            -1 {
              put $resp1 $dest1 "Bots don't have a pack level."
            }
          }
        } else {
          switch -exact -- [userStatusData $user list status] {
            pack {
              put $resp1 $dest1 "Level for $user: [userStatusData $user list level]"
            }
            assessment {
              put $resp1 $dest1 "$user is under assessment."
            }
            default {
              put $resp1 $dest1 "$user is not pack."
            }
          }
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Show a user's info: pub
  ##
  ## Args: nick, user@host, handle, channel, arg
  ## Returns: 1
  ##
  proc pub:uinfo {nick uhost hand chan arg} {
    return [uinfo pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  ##
  ## Show a user's info: msg
  ##
  ## Args: nick, user@host, handle, arg
  ## Returns: 1
  ##
  proc msg:uinfo {nick uhost hand arg} {
    return [uinfo msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Show a user's info: dcc
  ##
  ## Args: nick, user@host, handle, arg
  ## Returns: 1
  ##
  proc dcc:uinfo {hand idx arg} {
    return [uinfo dcc putdcc $idx putdcc $idx "" "" $hand [lindex [console $idx] 0] $arg]
  }

  ##
  ## Show a user's info
  ##
  ## Args: nick, user@host, handle, arg
  ## Returns: 1
  ##
  proc uinfo {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {((![string compare dcc $type]) || \
         ([floodCheck $nick $uhost uinfo])) && \
        ([validHandle $hand])} then {
      set who [lindex [split [string trimright $arg]] 0]
      if {[string length $who]} then {
        if {[matchattr $hand m|m $chan]} then {
          set user [findUser $who $chan]
        } else {
          put $resp2 $dest2 "Only masters can specify other users."
        }
      } else {
        set user $hand
      }
      if {[set result [checkUser $user]] != 1} then {
        switch -exact -- $result {
          0 {
            put $resp1 $dest1 "I don't know who $who is."
          }
          -1 {
            put $resp1 $dest1 "Bots don't have info."
          }
        }
      } else {
        switch -exact -- [userStatusData $user list status] {
          pack {
            set reply "HP \[[userPointData $user list hp]\]; FP \[[userPointData $user list fp]\]; Level \[[userStatusData $user list level]\]; Position \[[userStatusData $user list pos]\]"
          }
          assessment {
            switch -exact -- [set timeleft [userAssessmentData $user timeleft]] {
              -1 {
                set reply "Assessment started: [userAssessmentData $user started]"
              }
              0 {
                set reply "Assessment over"
              }
              default {
                set reply "Assessment time left: [duration $timeleft]"
              }
            }
          }
          default {
            if {![string compare $hand $user]} then {
              if {[string length $nick]} then {
                set uwho $nick
              } else {
                set uwho $hand
              }
              put $resp1 $dest1 "Sorry $uwho, but I don't have any info for you."
            } else {
              put $resp1 $dest1 "I don't have any info for $user."
            }
          }
        }
        if {[info exists reply]} then {
          put $resp1 $dest1 "Info for $user: $reply"
        }
      }
      return 1
    }
    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc"
    }
    foreach type $what {
      switch -exact -- $type {
       pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub m|m ${char}addpack ${NamespaceCurrent}::pub:addpack
          $action pub m|m ${char}delpack ${NamespaceCurrent}::pub:delpack
          $action pub m|m ${char}listpack ${NamespaceCurrent}::pub:listpack
          $action pub m|m ${char}addas ${NamespaceCurrent}::pub:addas
          $action pub m|m ${char}delas ${NamespaceCurrent}::pub:delas
          $action pub m|m ${char}listas ${NamespaceCurrent}::pub:listas
          $action pub m|m ${char}chpos ${NamespaceCurrent}::pub:chpos
          $action pub m|m ${char}listpos ${NamespaceCurrent}::pub:listpos
          $action pub m|m ${char}chlev ${NamespaceCurrent}::pub:chlev
          $action pub m|m ${char}listlev ${NamespaceCurrent}::pub:listlev
          $action pub -|- ${char}uinfo ${NamespaceCurrent}::pub:uinfo
        }
        msg {
          $action msg m|m addpack ${NamespaceCurrent}::msg:addpack
          $action msg m|m delpack ${NamespaceCurrent}::msg:delpack
          $action msg m|m listpack ${NamespaceCurrent}::msg:listpack
          $action msg m|m addas ${NamespaceCurrent}::msg:addas
          $action msg m|m delas ${NamespaceCurrent}::msg:delas
          $action msg m|m listas ${NamespaceCurrent}::msg:listas
          $action msg m|m chpos ${NamespaceCurrent}::msg:chpos
          $action msg m|m listpos ${NamespaceCurrent}::msg:listpos
          $action msg m|m chlev ${NamespaceCurrent}::msg:chlev
          $action msg m|m listlev ${NamespaceCurrent}::msg:listlev
          $action msg -|- uinfo ${NamespaceCurrent}::msg:uinfo
        }
        dcc {
          $action dcc m|m addpack ${NamespaceCurrent}::dcc:addpack
          $action dcc m|m delpack ${NamespaceCurrent}::dcc:delpack
          $action dcc m|m listpack ${NamespaceCurrent}::dcc:listpack
          $action dcc m|m addas ${NamespaceCurrent}::dcc:addas
          $action dcc m|m delas ${NamespaceCurrent}::dcc:delas
          $action dcc m|m listas ${NamespaceCurrent}::dcc:listas
          $action dcc m|m chpos ${NamespaceCurrent}::dcc:chpos
          $action dcc m|m listpos ${NamespaceCurrent}::dcc:listpos
          $action dcc m|m chlev ${NamespaceCurrent}::dcc:chlev
          $action dcc m|m listlev ${NamespaceCurrent}::dcc:listlev
          $action dcc -|- uinfo ${NamespaceCurrent}::dcc:uinfo
        }
      }
    }
    return
  }

} ;# namespace wrpg-statuscmds

return
