###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wrpgfserv.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpgfserv
# version: 0.6.3.3
# config: wrpgfserv.conf
# description:
# requires: this_module_needs_to_be_converted

namespace eval wrpgfserv {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # list the avaliable sound files; pub
  proc pub:sounds {nick uhost hand chan arg} {
    return [sounds pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # list the avaliable sound files; msg
  proc msg:sounds {nick uhost hand arg} {
    return [sounds msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # list the avaliable sound files
  proc sounds {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    global botnick files-path
    variable conf
    if {[validNick $nick]} then {
      set count 0
      foreach i [getfiles $conf(soundpath)] {
        lappend list "$i ([convert_size [file size [file join ${files-path} $conf(soundpath) $i]]]),"
        incr count
      }
      if {[info exists list]} then {
        formatLists $resp1 $dest1 [string trimright [join $list] ,] 160 , , " " "Sound files avaliable ($count): " "(cont): "
        put $resp1 $dest1 "To have any of the files sent to you, type: /msg $botnick send <filename>"
      } else {
        put $resp1 $dest1 "No sound files are currently avaliable."
      }
      return 1
    }
    return 0
  }

  # send user a sound file when requested; pub
  proc pub:send {nick uhost hand chan arg} {
    return [send pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # send user a sound file when requested; msg
  proc msg:send {nick uhost hand arg} {
    return [send msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # send user a sound file when requested
  proc send {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    global files-path
    variable conf
    variable NamespaceCurrent
    if {[validNick $nick]} then {
      set file [lindex [file split [lindex [split [string trimright $arg]] 0]] end]
      set who [lindex $arg 1]
      if {![string compare "" $file]} then {
        put $resp2 $dest2 "[command_char $type]send <file> \[nick\]"
      } else {
        if {![string compare "" $who]} then {
          set who $nick
        }
        if {([info exists files-path]) && ([info exists conf(soundpath)])} then {
          set filename [file join ${files-path} $conf(soundpath) $file]
          if {[file exists $filename]} then {
            wpLog x * $NamespaceCurrent "Sending `[file tail $filename]' to $who."
            dccsend $filename $who
          } elseif {![string compare $nick $who]} then {
            wpLog x * $NamespaceCurrent "Error: $nick!$uhost requested `$file', but it does not exist."
          } else {
            wpLog x * $NamespaceCurrent "Error: $nick!$uhost requested `$file' be sent to $who, but it does not exist."
          }
        }
      }
      return 1
    }
    return 0
  }

  # send out the rules file when requested; pub
  proc pub:rules {nick uhost hand chan arg} {
    return [rules pub putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # send out the rules file when requested; msg
  proc msg:rules {nick uhost hand arg} {
    return [rules msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # send out the rules file when requested
  proc rules {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    if {[validNick $nick]} then {
      global files-path
      variable conf
      variable NamespaceCurrent
      set file [file join ${files-path} $conf(textdir) $conf(rulesfile)]
      if {[file exists $file]} then {
        wpLog x * $NamespaceCurrent "Sending [file tail $file] to $nick."
        dccsend $file $nick
      } else {
        wpLog x * $NamespaceCurrent "Error: Rules file [file tail $file] does not exist."
      }
      return 1
    }
    return 0
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg"
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          $action pub -|- ${char}sounds ${NamespaceCurrent}::pub:sounds
          $action pub -|- !nick ${NamespaceCurrent}::pub:send
          $action pub -|- ${char}send ${NamespaceCurrent}::pub:send
          $action pub -|- ${char}rules ${NamespaceCurrent}::pub:rules
        }
        msg {
          $action msg -|- sounds ${NamespaceCurrent}::msg:sounds
          $action msg -|- $nick ${NamespaceCurrent}::msg:send
          $action msg -|- send ${NamespaceCurrent}::msg:send
          $action msg -|- rules ${NamespaceCurrent}::msg:rules
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    rulesfile rules.txt
    soundpath wolfpack/sounds/
    textdir text/
  }

  # Configuration data descriptions
  array set configDataDesc {
    rulesfile "\[file\] filename for rules in textdir"
    soundpath "\[dir\] directory for sound files (relative to files-path)"
    textdir "\[dir\] directory for text files (relative to files-path)"
  }

} ;# namespace wrpgfserv

return
