###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wrpgpers.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpgpers
# version: 0.6.1.2
# config: wrpgpers.conf
# description:
# requires: this_module_needs_to_be_converted
# provides: personality

namespace eval wrpgpers {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  ########################################
  #FIXME: temporarily hard coded

  set persdata(botnick) {
    "%act %chan blinks"
    "%act %chan looks up at %nick"
    "%act %chan perks an ear"
    "%act %chan bats an eyelash"
    "%act %chan wags her tail"
  }
  set persdata(ownerjoin) {
    "%act %chan welcomes %nick, her caretaker and friend."
  }
  set persdata(part) {
    "%not %nick Be safe, my friend"
  }
  set persdata(packjoin) {
    "%snd %chan %sound:bark stands and yips with joy as %nick enters."
    "%act %chan pounces %nick and slobbers all over the poor wolf!"
    "%snd %chan %sound:bark barks as she sees %nick come in."
    "%act %chan leaps up and runs in a circle, estatic to see %nick."
    "%act %chan rolls over, hoping for a tummy rub from %nick."
    "%act %chan nuzzles %nick softly, welcome home."
    "%act %chan blinks and flips her tail back and forth as she sees %nick enter."
    "%act %chan rolls onto her back and looks at %nick from a different point of view."
  }
  set persdata(packpart) {
    "%snd %chan %sound:whine whines as %nick leaves"
  }
  set persdata(packsign) {
    "%snd %chan %sound:whine whines as %nick leaves"
  }
  set persdata(packhowl) {
    "%snd %chan %sound:howl hears howling and joins in"
  }
  set persdata(identify) {
    "%not %nick Do I look like a ChanServ to you!?"
    "%not %nick There is no command named 'identify', perhaps you mean 'ident'?"
  }
  set persdata(huntkill) {
    "%snd %chan %sound:howl sings with her family for the spirit of the %animal"
  }
#FIXME: moonphase hooks
#  set persdata(fullmoon) {
#    "%snd %chan %sound:howl howls at the full moon"
#  }


  ########################################

  # give the bot an attitude of its own ;)
  proc personality {what args} {
    variable persdata
    set string [lindex $persdata($what) [rand [llength $persdata($what)]]]
    set type [lindex $string 0]
    set result [lrange $string 1 end]
    foreach i $args {
      regsub -all -- [lindex $i 0] $result [lindex $i 1] result
    }
    switch -exact $type {
      %not {
        putnot [lindex $result 0] [lrange $result 1 end]
      }
      %msg {
        putmsg [lindex $result 0] [lrange $result 1 end]
      }
      %act {
        putact [lindex $result 0] [lrange $result 1 end]
      }
      %snd {
        set result [substitute_sound $result]
        putsnd [lindex $result 0] [lrange $result 1 end]
      }
    }
    return
  }

  # respond to out own nickname being said
  proc pubm:botnick {nick uhost hand chan arg} {
    if {([validNick $nick]) && ([flood_check $nick])} then {
      personality botnick "%nick $nick" "%chan $chan"
    }
    return 0
  }

  # respond to out own nickname being said
  proc ctcp:action_botnick {nick uhost hand dest key arg} {
    global botnick
    if {([validNick $nick]) && ([flood_check $nick])} then {
      if {[regexp ".*[string toupper $botnick].*" [string toupper $arg]]} then {
        personality botnick "%nick $nick" "%chan $dest"
      }
    }
    return 0
  }

  # send greet for owners on join
  proc join:owner {nick uhost hand chan} {
    if {[validNick $nick]} then {
      personality ownerjoin "%nick $nick" "%chan $chan"
    }
    return 0
  }

  # send notice on part
  proc part:personality {nick uhost hand chan} {
    if {[validNick $nick]} then {
      personality part "%nick $nick" "%chan $chan"
    }
    return 0
  }

  # send greet on join
  proc join:pack {nick uhost hand chan} {
    if {[validNick $nick]} then {
      personality packjoin "%nick $nick" "%chan $chan"
    }
    return 0
  }

  # send a sound on part
  proc part:pack {nick uhost hand chan} {
    if {[validNick $nick]} then {
      if {[llength [chanlist $chan]] > 1} then {
        personality packpart "%nick $nick" "%chan $chan"
      }
    }
    return 0
  }

  # send a sound on signoff
  proc sign:pack {nick uhost hand chan arg} {
    if {[validNick $nick]} then {
      if {[llength [chanlist $chan]] > 1} then {
        personality packsign "%nick $nick" "%chan $chan"
      }
    }
    return 0
  }

  # howl along with the channel
  proc pubm:howl {nick uhost hand chan arg} {
    if {([validNick $nick]) && ([flood_check $nick])} then {
      personality packhowl "%nick $nick" "%chan $chan"
    }
    return 0
  }

  # howl along with the channel
  proc ctcp:action_howl {nick uhost hand dest key arg} {
    if {([validNick $nick]) && ([flood_check $nick])} then {
      if {([regexp ".*ho.*ow.*ww.*lll.*" [string tolower $arg]]) || \
          ([regexp ".*ar.*ro.*oo.*" [string tolower $arg]])} then {
        personality packhowl "%nick $nick" "%chan $chan"
      }
    }
    return 0
  }

  # send a notice to users who use chanserv too much
  proc msg:identify {nick uhost hand arg} {
    variable NamespaceCurrent
    if {([validHandle $nick]) && ([validNick $nick])} then {
        personality identify "%nick $nick"
    }
    wpLog c * $NamespaceCurrent "($nick!$uhost) !$hand! identify"
    return 0
  }

  # notice a user when pinged
  proc ctcp:ping {nick uhost hand dest key arg} {
    putnot $nick "Pong!"
    return 0
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "msg pubm join part sign ctcp"
    }
    foreach i $what {
      switch -exact -- $i {
        msg {
          $action msg -|- identify ${NamespaceCurrent}::msg:identify
        }
        pubm {
          $action pubm -|- *nick* ${NamespaceCurrent}::pubm:botnick
          $action pubm P|P #*~*ho*ow*ww*lll* ${NamespaceCurrent}::pubm:howl
          $action pubm P|P #*~*ar*ro*oo* ${NamespaceCurrent}::pubm:howl
        }
        join {
          $action join n|- * ${NamespaceCurrent}::join:owner
          $action join P|P * ${NamespaceCurrent}::join:pack
        }
        part {
          $action part -|- * ${NamespaceCurrent}::part:personality
          $action part P|P * ${NamespaceCurrent}::part:pack
        }
        sign {
          $action sign P|P * ${NamespaceCurrent}::sign:pack
        }
        ctcp {
          $action ctcp -|- ACTION ${NamespaceCurrent}::ctcp:action_botnick
          $action ctcp P|P ACTION ${NamespaceCurrent}::ctcp:action_howl
          $action ctcp -|- PING ${NamespaceCurrent}::ctcp:ping
        }
      }
    }
    return
  }

} ;# namespace wrpgpers

return
