###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: wrpgsound.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: wrpgsound
# version: 0.6.5.3
# config: wrpgsound.conf
# description:
# requires: this_module_needs_to_be_converted
# provides: valid_type random_sound substitute_sound

namespace eval wrpgsound {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # set data information
  set data(sounddata) "1 $conf(sounddatafile) ${NamespaceCurrent}::sounddata"

  # load data
  module_data load "" 1 0 "1 $conf(sounddatafile) ${NamespaceCurrent}::sounddata"

  # refresh sound data; dcc
  proc dcc:refsound {hand idx arg} {
    variable NamespaceCurrent
    set type [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $type]} then {
      wpLog o * $NamespaceCurrent "Refreshing sound data..."
      refsound ""
    } else {
      wpLog o * $NamespaceCurrent "Refreshing sound data for type `$type'..."
      refsound $type
    }
    return 1
  }

  # refresh sound data; fil
  proc fil:refsound {hand idx arg} {
    variable NamespaceCurrent
    set type [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $type]} then {
      wpLog o * $NamespaceCurrent "Refreshing sound data..."
      refsound ""
    } else {
      wpLog o * $NamespaceCurrent "Refreshing sound data for type `$type'..."
      refsound $type
    }
    return 1
  }

  #FIXME: do this on a time binding
  # refresh the sound data
  proc refsound {type} {
    variable conf
    variable sounddata
    variable sounddata_changed
    if {![string compare "" $type]} then {
      set update $conf(types)
    } elseif {[valid_type $type]} then {
      set update $type
    }
    if {[info exists update]} then {
      foreach i $update {
        set list ""
        foreach j [lsort [getfiles $conf(soundpath)]] {
          set desc [string tolower [getdesc $conf(soundpath) $j]]
          if {[regexpp "%$i.*" $desc]} then {
            lappend list $j
          }
        }
        if {[info exists list]} then {
          set sounddata($i) [string trimright [join $list]]
        } elseif {[info exists sounddata($i)]} then {
          unset sounddata($i)
        }
      }
      set sounddata_changed 1
      return 1
    }
    return 0
  }

  # returns 1 if the given sound type is valid
  proc valid_type {type} {
    variable conf
    if {[lsearch -exact $conf(types) $type] != -1} then {
      return 1
    }
    return 0
  }

  # returns a random sound of the given type
  proc random_sound {type} {
    variable sounddata
    if {[info exists sounddata($type)]} then {
      return [lindex $sounddata($type) [rand [llength $sounddata($type)]]]
    }
    return ""
  }

  # do sound substitutions for the given string
  proc substitute_sound {string} {
    if {[regexp ".*%sound:.*" $string]} then {
      for {set index 0} {$index < [llength $string]} {incr index} {
        if {[regexp "%sound:.*" [lindex $string $index]]} then {
          set type [lindex [split [lindex $string $index] :] 1]
          set sound [random_sound $type]
          if {[string compare "" $sound]} then {
            set string [lreplace $string $index $index $sound]
          } else {
            set string [lreplace $string $index $index ${type}.wav]
          }
        }
      }
    }
    return $string
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "dcc fil"
    }
    foreach i $what {
      switch -exact -- $i {
        dcc {
          $action dcc M|M refsound ${NamespaceCurrent}::dcc:refsound
        }
        fil {
          $action fil M|M refsound ${NamespaceCurrent}::fil:refsound
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    sounddatafile wrpgsound-cache.data
    soundpath wolfpack/sounds/
    types "bark howl whine"
  }

  # Configuration data descriptions
  array set configDataDesc {
    sounddatafile "\[file\] file that cached sound data is stored in"
    soundpath "\[path\] pathname within files-path for sound files"
    types "\[types\] sound types to search file descriptions for"
  }

} ;# namespace wrpgsound

return
