#! /bin/sh
# \
# Nice little hack to find latest version of tclsh in PATH \
# \
# NOTE: backslash and semicolon placements are important! \
# \
# Search for tclsh[0-9].[0-9] in each valid dir in PATH \
for dir in $(echo $PATH | sed 's/:/ /g'); \
do \
  if test -d $dir; \
  then \
    files=$(/bin/ls $dir | egrep '^tclsh[0-9]\.[0-9]$'); \
    if test "$files" != ""; \
    then \
      versions="${versions:+$versions }$(echo $files | sed 's/tclsh//g')"; \
    fi; \
  fi; \
done; \
# Loop over each version to find the latest version of tclsh \
for ver in $versions; \
do \
  tmpver=$(echo $ver | sed 's/\.//g'); \
  if test "$lasttmpver" != ""; \
  then \
    if test "$tmpver" -gt "$lasttmpver"; \
    then \
      lastver=$ver; \
      lasttmpver=$tmpver; \
    fi; \
  else \
    lastver=$ver; \
    lasttmpver=$tmpver; \
  fi; \
done; \
# Use the latest tclsh version found, otherwise fall back to 'tclsh' \
exec tclsh$lastver "$0" "$@"
###############################################################################
##
##  Wolfpack - A modular Tcl system for Eggdrop 1.3.0+ with Tcl 8.0+
##  Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
###############################################################################
##
##  You should not need to edit anything in any of these files.
##
##  './wolfpack.tcl -c' will allow you to configure initial settings.
##  './wolfpack.tcl --help' will list all avaliable options.
##
##  Use '.wpconf' in dcc chat with the bot to set everything else.
##
###############################################################################
##
## $Id: wolfpack.tcl,v 1.6 2001/06/04 04:52:40 tothwolf Exp $
##

# make sure Tcl version is compatible with this code (we use namespaces)
if {[catch {package require Tcl 8.0}]} then {
  if {[info exists argv0]} then {
    puts "Wolfpack: Error: wolfpack requires Tcl 8.0 or later to load."
  } else {
    putloglev o * "Wolfpack: Error: wolfpack requires Tcl 8.0 or later to load."
  }
  return 0
}

namespace eval :: {

  # Eggdrop doesn't currently set argv0, so we use that to detect load type.
  global argv0

  if {![info exists argv0]} then {
    # quick hack for buggy tcl variables in older bots
    catch {set version}
    catch {set numversion}
    # require eggdrop 1.3.0 or later
    if {(![info exists version]) || (![info exists numversion]) || \
        ($numversion < 1030000)} then {
      putloglev o * "Wolfpack: Error: wolfpack requires Eggdrop 1.3.0 or later to load."
      return 0
    }
  } else {
    # emulate eggdrop's putloglev when loading with tclsh
    if {![string compare "" [info commands putloglev]]} then {
      proc putloglev {level channel text} {
        puts $text
      }
    }
  }

  ##
  ## Log tcl messages where they can be seen
  ##
  ## Args:    text
  ## Returns: nothing
  ##
  proc tclLog {text} {
    # Tcl's tclLog embeds newlines in it's output
    if {[string first \n $text] == -1} then {
      putloglev o * "Wolfpack: \[Tcl\]: $text"
    } else {
      foreach line [split $text \n] {
        putloglev o * "Wolfpack: \[Tcl\]: $line"
      }
    }
    return
  }

} ;# namespace ::

namespace eval ::wp {

  # manage package information
  package forget ::wp
  package provide ::wp 1.9.9.0

  # set namespace variable
  set NamespaceCurrent [namespace current]

  ##
  ## WARNING: If you change these, you may render your module database useless!
  ##

  # Max number of lines to scan in a module file
  set moduleDatabaseConfig(scanlines) 30

  # Max depth to list directories in module path
  set moduleDatabaseConfig(maxdepth) 4

  # Module database version
  set moduleDatabaseConfig(version) 2.0

  # Module database header
  set moduleDatabaseConfig(header) "Wolfpack module database "

  # Module database defaults
  set moduleDatabaseConfig(defaults) "{version 0.1} {description {(no description)}} {load 0}"

  # Versioned module database formats
  array set moduleDatabaseFormat {
    2.0 "{version config help description provides requires} {load file md5sum}"
  }

  # md5 style, command name and result string index
  array set md5Format {
    bsd "md5 3"
    gnu "md5sum 0"
  }

  # Configuration file
  set configDefaults(cfgfile) wolfpack.conf

  # Interactive configuration prompt
  set configDefaults(prompt) "wolfpack> "

  # Configuration data defaults
  array set configDataDefaults {
    modulepath modules/
    configpath wpconf/
    datapath wpdata/
    helppath wphelp/
    moddbfile wolfpack.db
    rebuild 1
    verbose 0
    time 0
  }

  # Configuration data descriptions
  array set configDataDesc {
    modulepath "Module path"
    configpath "Config path"
    datapath "Data path"
    helppath "Help path"
    moddbfile "Module database"
    rebuild "Automatically rebuild database"
    verbose "Verbose operation"
    time "Time database compare and rebuild"
  }

  # Command line option defaults
  array set optionData {
    config 0
    rebuild 0
    norebuild 0
    time 0
    verbose 0
    quiet 0
    debug 0
  }

  # Exported commands
  set ExportList {
    md5Sum
    md5Init
    replaceExpr
    listFiles
    listSubdirs
    findFiles
    createFile
    createDir
    backupFile
    listSave
    listLoad
    arraySave
    arrayLoad
    arraySetAll
    arrayFindElementName
    arrayMaxElementDataLength
    listMaxElementLength
    splitList
    arraySearch
    dataFormatDefault
    dataFormatValue
    dataFormatReplace
    dataFormatBuild
    dataFormatConvert
    scanModule
    getModuleDatabaseData
    setModuleDatabaseData
    saveModuleDatabase
    loadModuleDatabase
    compareModuleDatabase
    rebuildModuleDatabase
    listModules
    moduleExists
    moduleLoaded
    moduleLoad
    moduleUnload
    moduleConfig
    moduleConfigList
    moduleData
    moduleDataList
    buildCommandTable
    whichCommand
    whichModule
    whichModuleCommand
    compareVersion
    wpLog
  }

  ##
  ## Create md5 checksum for a file
  ##
  ## Args:    filename
  ## Returns: md5 checksum if successful
  ## Errors:  permission denied,
  ##          no such file,
  ##          not a file,
  ##          can't exec md5 command
  ##
  proc md5Sum {file} {
    variable md5Config

    if {![file exists $file]} then {
      error "$file: no such file"
    } else {
      if {![file isfile $file]} then {
        error "$file: not a file"
      } else {
        if {![file readable $file]} then {
          error "$file: permission denied"
        } else {
          if {[catch {set sum [lindex [exec $md5Config(command) $file] $md5Config(index)]} result]} then {
            error "$file: $result"
          } else {
            return $sum
          }
        }
      }
    }
  }

  ##
  ## Init md5 command
  ##
  ## Args:    none
  ## Returns: 1 if a useable md5 command found
  ##          0 otherwise
  ##
  proc md5Init {} {
    variable md5Config
    variable md5Format

    foreach type [array names md5Format] {
      foreach {command index} $md5Format($type) {break}
      if {([catch {exec $command ""} result]) && \
          (![regexp -- "^couldn't execute" $result])} then {
        set md5Config(command) $command
        set md5Config(index) $index
        return 1
      }
    }
    return 0
  }

  ##
  ## Replace all occurances of an expression in a string with the given text
  ##
  ## Args:    string, expr, replacement text
  ## Returns: string
  ##
  proc replaceExpr {string expr {replace ""}} {
    while {[regexp -nocase -- $expr $string]} {
      regsub -all -- $expr $string $replace string
    }
    return $string
  }

  ##
  ## List files in a path
  ##
  ## Args:    path
  ## Returns: list of files in the given path,
  ##          nothing if no files in the given path
  ## Errors:  permission denied,
  ##          no such directory,
  ##          not a directory
  ##
  proc listFiles {path} {
    if {![file exists $path]} then {
      error "$path: no such directory"
    } else {
      if {![file isdirectory $path]} then {
        error "$path: not a directory"
      } else {
        if {![file readable $path]} then {
          error "$path: permission denied"
        } else {
          set ret ""
          foreach name [lsort [glob -nocomplain -- [file join $path *]]] {
            if {[file isfile $name]} then {
              lappend ret $name
            }
          }
          return $ret
        }
      }
    }
  }

  ##
  ## List subdirs in a path
  ##
  ## Args:    path
  ## Returns: list of subdirs in the given path,
  ##          nothing if no subdirs in the given path
  ## Errors:  permission denied,
  ##          no such directory,
  ##          not a directory
  ##
  proc listSubdirs {path} {
    if {![file exists $path]} then {
      error "$path: no such directory"
    } else {
      if {![file isdirectory $path]} then {
        error "$path: not a directory"
      } else {
        if {![file readable $path]} then {
          error "$path: permission denied"
        } else {
          set ret ""
          foreach name [lsort [glob -nocomplain -- [file join $path *]]] {
            if {[file isdirectory $name]} then {
              lappend ret $name
            }
          }
          return $ret
        }
      }
    }
  }

  ##
  ## List files with a set ext in a path and its subdirs up to a set depth
  ##
  ## Args:    path, max search depth, file extension
  ## Returns: list of files with a set ext in the given path and its subdirs,
  ##          nothing if no matching files are found
  ##
  proc findFiles {path depth {ext ""}} {
    set ret ""
    set foundDirs "$path "
    set searchDirs $path
    for {
      set currentDepth 0
    } {($currentDepth <= $depth) || (!$depth)} {
      incr currentDepth
    } {
      set subDirs ""
      foreach dir $searchDirs {
        if {[catch {set dirList [listSubdirs $dir]} result]} then {
          wpLog o * "Error: unable to get file listing: $result"
        } elseif {[string compare "" $dirList]} then {
          append subDirs $dirList " "
        }
      }
      if {![string compare "" $subDirs]} then {
        break
      }
      append foundDirs $subDirs " "
      set searchDirs $subDirs
    }
    foreach dir $foundDirs {
      if {[catch {set files [listFiles $dir]} result]} then {
        wpLog o * "Error: unable to get file listing: $result"
      } else {
        if {[string compare "" $ext]} then {
          foreach file $files {
            if {![string compare $ext \
                  [string tolower [file extension $file]]]} then {
              lappend ret $file
            }
          }
        } else {
          set ret $files
        }
      }
    }
    return $ret
  }

  ##
  ## Check if a file exists, and create it if not
  ##
  ## Args:    filename, verbose {-1,0,1}, description, force new file
  ## Returns: 1 if the file was created successfully
  ##          0 if the operation failed
  ##         -1 if the file already exists
  ##
  proc createFile {file {verbose 0} {desc "file "} {force 0}} {
    if {($force) || (![file exists $file])} then {
      if {[catch {set fd [open $file w]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to create ${desc}`[file tail $file]': $result"
        }
      } else {
        if {(!$force) && ($verbose >= 1)} then {
          wpLog o * "Warning: ${desc}`[file tail $file]' does not exist -- creating"
        }
        close $fd
        return 1
      }
    } elseif {[file isfile $file]} then {
      return -1
    } elseif {$verbose >= 0} then {
      wpLog o * "Error: not a file: $file"
    }
    return 0
  }

  ##
  ## Check if a directory exists, and create it if not
  ##
  ## Args:    directory, verbose {-1,0,1}, description
  ## Returns: 1 if the directory was created successfully
  ##          0 if the operation failed
  ##         -1 if the directory already exists
  ##
  proc createDir {dir {verbose 0} {desc "directory "}} {
    if {![file exists $dir]} then {
      if {[catch {file mkdir $dir} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to create ${desc}`[file tail $dir]': $result"
        }
      } else {
        if {$verbose >= 1} then {
          wpLog o * "Warning: ${desc}`[file tail $dir]' does not exist -- creating"
        }
        return 1
      }
    } elseif {[file isdirectory $dir]} then {
      return -1
    } elseif {$verbose >= 0} then {
      wpLog o * "Error: not a directory: $dir"
    }
    return 0
  }

  ##
  ## Create a backup of the given file with an optional suffix
  ##
  ## Args:    filename, suffix, verbose {-1,0,1}
  ## Returns:  1 if successful
  ##          -1 if file is 0 in size
  ##           0 otherwise
  ##
  proc backupFile {file {verbose 0} {suffix ~bak}} {
    variable NamespaceCurrent

    if {[string compare "" $suffix]} then {
      if {[file size $file]} then {
        if {[catch {
          file copy -force $file $file${suffix}
        } result]} then {
          if {$verbose >= 0} then {
            wpLog o * $NamespaceCurrent "Error: unable to create backup file for `[file tail $file]': $result"
          }
        } else {
          return 1
        }
      } else {
        return -1
      }
    }
    return 0
  }

  ##
  ## Save data from a list into a file
  ##
  ## Args:    list, filename, verbose {-1,0,1}, description, access flag
  ## Returns: 1 is successful,
  ##          0 otherwise
  ##
  proc listSave {listName file {verbose 0} {desc "file "} {access w}} {
    upvar 1 $listName list

    if {[createFile $file $verbose $desc]} then {
      if {[catch {set fd [open $file $access]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to open ${desc}`$file' for writing: $result"
        }
      } else {
        if {[info exists list]} then {
          foreach data $list {
            puts $fd [list $data]
          }
        }
        close $fd
        return 1
      }
    }
    return 0
  }

  ##
  ## Load data into a list from a file
  ##
  ## Args:    list, filename, verbose {-1,0,1}, description, ignore regsub
  ## Returns: 1 if successful,
  ##          0 otherwise
  ##
  proc listLoad {listName file {verbose 0} {desc "file "} {ignore "^#"}} {
    upvar 1 $listName list

    if {[createFile $file $verbose $desc]} then {
      if {[catch {set fd [open $file r]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to open ${desc}`$file' for reading: $result"
        }
      } else {
        if {[info exists list]} then {
          unset list
        }
        while {![eof $fd]} {
          set line [replaceExpr [gets $fd] "^ "]
          if {([string compare "" $line]) && \
              (![regexp -- $ignore $line])} then {
            append list $line " "
          }
        }
        close $fd
        return 1
      }
    }
    return 0
  }

  ##
  ## Save data from an array info a file
  ##
  ## Args:    array, filename, verbose {-1,0,1}, description, access flag
  ## Returns: 1 is successful,
  ##          0 otherwise
  ##
  proc arraySave {arrayName file {verbose 0} {desc "file "} {access w}} {
    upvar 1 $arrayName array

    if {[createFile $file $verbose $desc]} then {
      if {[catch {set fd [open $file $access]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to open ${desc}`$file' for writing: $result"
        }
      } else {
        if {[array exists array]} then {
          foreach name [lsort [array names array]] {
            puts $fd "[list $name] [list $array($name)]"
          }
          close $fd
          return 1
        } else {
          close $fd
        }
      }
    }
    return 0
  }

  ##
  ## Load data into an array from a file
  ##
  ## Args:    array, filename, verbose {-1,0,1}, description, ignore regsub
  ## Returns: 1 if successful,
  ##          0 otherwise
  ##
  proc arrayLoad {arrayName file {verbose 0} {desc "file "} {ignore "^#"}} {
    upvar 1 $arrayName array

    if {[createFile $file $verbose $desc]} then {
      if {[catch {set fd [open $file r]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to open ${desc}`$file' for reading: $result"
        }
      } else {
        if {[info exists array]} then {
          unset array
        }
        while {![eof $fd]} {
          set line [replaceExpr [gets $fd] "^ "]
          if {([string compare "" $line]) && \
              (![regexp -- $ignore $line])} then {
            set array([lindex $line 0]) [lindex $line 1]
          }
        }
        close $fd
        return 1
      }
    }
    return 0
  }

  ##
  ## Set all elements in the given array the the given value
  ##
  ## Args:    array name, value
  ## Returns: 1 if the array exists
  ##          0 otherwise
  ##
  proc arraySetAll {arrayName {value ""}} {
    upvar 1 $arrayName array

    if {[array exists array]} then {
      foreach name [array names array] {
        set array($name) $value
      }
      return 1
    }
    return 0
  }

  ##
  ## Find the given element in an array
  ##
  ## Args:    array name, element name
  ## Returns: case sensitive element name if found,
  ##          nothing otherwise
  ##
  proc arrayFindElementName {array element} {
    set list [lsort [array names $array]]
    set index [lsearch -exact [string tolower $list] [string tolower $name]]
    if {$index != -1} then {
      return [lindex $list $index]
    }
    return
  }

  ##
  ## Return length of longest data in an array
  ##
  ## Args:    array name
  ## Returns: length of longest name in an array
  ##
  proc arrayMaxElementDataLength {arrayName} {
    upvar 1 $arrayName array

    set maxlength 0
    foreach {name data} [array get array] {
      set length [string length $data]
      if {$length > $maxlength} then {
        set maxlength $length
      }
    }
    return $maxlength
  }

  ##
  ## Return length of the longest element in a list
  ##
  ## Args:    list
  ## Returns: length of longest element in the given list
  ##
  proc listMaxElementLength {list} {
    set maxlength 0
    foreach data $list {
      set length [string length $data]
      if {$length > $maxlength} then {
        set maxlength $length
      }
    }
    return $maxlength
  }

  ##
  ## Split up a list into multiple elements
  ##
  ## Args:    text, max list length, split char, trim chars
  ## Returns: split list
  ##
  # FIXME: improve this
  proc splitList {text {splitLength 75} {splitChar " "} {trimChars " "}} {
    # Sanity check splitLength and splitChar
    if {($splitLength >= 1) && ([string compare "" $splitChar])} then {
      set elementSplitLength [expr $splitLength - 1]
      set stringLength [string length $text]	;# Total length of string
      set subStringLength 0			;# Text left over
      set elementLength 0			;# Element length counter
      set elementStartIndex 0			;# Start of split element
      set elementEndIndex 0			;# End of split element
      for {
        set stringIndex 0
      } {$stringIndex < $stringLength} {
        incr stringIndex
      } {
        # If element length greater than/equal to split length,
        # Or element length equal to split length - 1,
        # And character at current string index is splitChar
        if {(($elementLength >= $splitLength) ||
             ($elementLength == $elementSplitLength)) &&
            (![string compare $splitChar \
               [string index $text $stringIndex]])} then {
          # Split substring element from text
          set string [string range $text $elementStartIndex $elementEndIndex]
          # Append substring element list to list
          lappend list [string trim $string $trimChars]
          # Reset element length counter
          set elementLength 0
          # Start split of next element at the end + 1 of the current one
          set elementStartIndex [expr $elementEndIndex + 1]
          # Reset end of next element to the start of the next element
          set elementEndIndex $elementStartIndex
          # Track remaining text length
          set subStringLength [expr $subStringLength + [string length $string]]
        } else {
          # Increment element length
          incr elementLength
          # Increment end of next element
          incr elementEndIndex
        }
      }
      # Append any left over text as a new element
      if {$stringLength > $subStringLength} then {
        lappend list [string trim [string range $text $subStringLength end] $trimChars]
      }
      # Whew...that was alot of work!
      if {[info exists list]} then {
        return $list
      }
    }
    return
  }

  ##
  ## Search an array for a given word or regexp
  ##
  ## Args:    array name, word/regexp
  ## Returns: list of indexes that match the given word/regexp
  ##
  proc arraySearch {array word} {
    set word [string tolower $word]
    set ret ""
    foreach {name data} [array get $array] {
      set string [string tolower $data]
      if {[lsearch -regexp $string $word] != -1} then {
        for {
          set index 0
          set indexes ""
        } {
          if {[regexp -- .*$word $string]} then {
            lappend indexes $index
          }
        } {
          incr index
        }
        lappend ret [list [concat $name $indexes]]
      }
    }
    return $ret
  }

  ##
  ## Find option default for the given option name in a data list
  ##
  ## Args:    data list, option name
  ## Returns: option default if found,
  ##          nothing otherwise
  ##
  proc dataFormatDefault {list option} {
    foreach i $list {
      if {![string compare $option [lindex $i 0]]} then {
        return [lindex $i 1]
      }
    }
    return
  }

  ##
  ## Find option value for the given option name in a data list
  ##
  ## Args:    data format, data list, option name
  ## Returns: option value if found,
  ##          nothing otherwise
  ##
  proc dataFormatValue {format data option} {
    if {[set index [lsearch -exact $format $option]] != -1} then {
      return [lindex $data $index]
    }
    return
  }

  ##
  ## Replace option data in the given data list with a new value
  ##
  ## Args:    data format, data list, option name, new value
  ## Returns: data list
  ##
  proc dataFormatReplace {format data option value} {
    if {[set index [lsearch -exact $format $option]] != -1} then {
      return [lreplace $data $index $index $value]
    }
    return $data
  }

  ##
  ## Create a data format list for a given data format and options
  ##
  ## Args:    data format list, defaults, options {{option1 value} ...}
  ## Returns: data format list with options and values in proper order
  ##
  proc dataFormatBuild {format defaults args} {
    set ret ""
    foreach arg $args {
      set [lindex $arg 0] [lindex $arg 1]
    }
    foreach opt $format {
      if {[info exists $opt]} then {
        lappend ret [set $opt]
      } else {
        lappend ret [dataFormatDefault $defaults $opt]
      }
    }
    return $ret
  }

  ##
  ## Convert a data list from one format to another
  ##
  ## Args:    from format, to format, data list
  ## Returns: data list
  ##
  proc dataFormatConvert {fromFormat toFormat data} {
    set ret ""
    set index 0
    foreach opt $fromFormat {
      set $opt [lindex $data $index]
      incr index
    }
    foreach opt $toFormat {
      if {[info exists $opt]} then {
        lappend ret [set $opt]
      } else {
        lappend ret [dataFormatDefault $defaults $opt]
      }
    }
    return $ret
  }

  ##
  ## Scan the given file for module options
  ##
  ## Args:    file, args {only scan for these options}
  ## Returns: list of module options if the given file is a module,
  ##          nothing otherwise
  ## Errors:  unable to open file for reading
  ##
  proc scanModule {file args} {
    variable moduleDatabaseConfig
    variable moduleDatabaseFormat

    if {[catch {set fd [open $file r]} result]} then {
      error $result
    } else {
      set ret ""
      if {![string compare "" $args]} then {
        set baseOptions [lindex $moduleDatabaseFormat([set moduleDatabaseConfig(version)]) 0]
        set extraOptions [lindex $moduleDatabaseFormat([set moduleDatabaseConfig(version)]) 1]
        set scanOptions "name $baseOptions"
        set formatOptions "name $baseOptions $extraOptions"
      } else {
        set scanOptions $args
        set formatOptions $args
      }
      for {
        set lineCount 0
        set optionCount 0
        set continuedLine 0
      } {(![eof $fd]) && ($lineCount <= $moduleDatabaseConfig(scanlines))} {
        incr lineCount
      } {
        gets $fd line
        if {[regexp -- "^# .*:.*" $line]} then {
          set opt [string trimright [lindex $line 1] :]
          if {[lsearch -glob $scanOptions $opt] != -1} then {
            set data [string trimright [string trimleft [string range $line [string first : $line] end] " \t:"] " \t\\"]
            if {![info exists $opt]} then {
              set $opt $data
            } else {
              append $opt " $data"
            }
          }
          if {[regexp -- \\\\$ $line]} then {
            set continuedLine 1
          } else {
            set continuedLine 0
          }
        } elseif {($continuedLine) && ([info exists opt])} then {
          append $opt " [string trimright [string trimleft $line " \t#"] " \t\\"]"
          if {![regexp -- \\\\$ $line]} then {
            set continuedLine 0
          }
        }
      }
      close $fd
      if {(![string compare "" $args]) && \
          ((![info exists name]) || \
           ([catch {set md5sum [md5Sum $file]}]))} then {
        return
      }
      foreach option $formatOptions {
        if {(![info exists $option]) || \
            (![string compare "" [set $option]])} then {
          set $option [dataFormatDefault $moduleDatabaseConfig(defaults) $option]
        }
        lappend ret [set $option]
      }
      return $ret
    }
  }

  ##
  ## Get data from module db data array
  ##
  ## Args:    module name, data type
  ## Returns: data for the given module's data type if it exists,
  ##          nothing otherwise
  ##
  proc getModuleDatabaseData {module type} {
    variable moduleDatabaseConfig
    variable moduleDatabaseFormat
    variable moduleDatabaseData

    if {[moduleExists $module]} then {
      set index [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] $type]
      if {$index != -1} then {
        return [lindex $moduleDatabaseData($module) $index]
      }
    }
    return
  }

  ##
  ## Set data in module db data array
  ##
  ## Args:    module name, data type, data
  ## Returns: 1 if valid module and data type,
  ##          0 otherwise
  ##
  proc setModuleDatabaseData {module type data} {
    variable moduleDatabaseConfig
    variable moduleDatabaseFormat
    variable moduleDatabaseData
    variable moduleDatabaseDataChanged

    if {[moduleExists $module]} then {
      set index [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] $type]
      if {$index != -1} then {
        set moduleDatabaseData($module) [lreplace $moduleDatabaseData($module) $index $index [list $data]]
        set moduleDatabaseDataChanged 1
        return 1
      }
    }
    return 0
  }

  ##
  ## Save module database
  ##
  ## Args:    verbose {-1,0,1}
  ## Returns: 1 if successful,
  ##          0 otherwise
  ##
  proc saveModuleDatabase {{verbose 0}} {
    variable configData
    variable moduleDatabaseConfig
    variable moduleDatabaseData
    variable moduleDatabaseDataChanged

    if {[createFile $configData(moddbfile) $verbose "module database file "]} then {
      if {[catch {set fd [open $configData(moddbfile) w]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to open module database file `$configData(moddbfile)' for writing: $result"
        }
      } else {
        puts $fd "# $moduleDatabaseConfig(header)$moduleDatabaseConfig(version)"
        close $fd
        set ret [arraySave moduleDatabaseData $configData(moddbfile) $verbose "module database file " a]
        if {$ret} then {
          set moduleDatabaseDataChanged 0
        }
        return $ret
      }
    }
    return 0
  }

  ##
  ## Load module database
  ##
  ## Args:    verbose {-1,0,1}
  ## Returns: 1 if successful,
  ##          0 otherwise
  ##
  proc loadModuleDatabase {{verbose 0}} {
    variable configData
    variable moduleDatabaseConfig
    variable moduleDatabaseFormat
    variable moduleDatabaseData
    variable moduleDatabaseDataChanged

    if {![file exists $configData(moddbfile)]} then {
      return -1
    } else {
      if {[catch {set fd [open $configData(moddbfile) r]} result]} then {
        if {$verbose >= 0} then {
          wpLog o * "Error: unable to open module database file `$configData(moddbfile)' for reading: $result"
        }
      } else {
        set firstline [replaceExpr [gets $fd] "^ "]
        if {[regexp -- "^# $moduleDatabaseConfig(header)" $firstline]} then {
          regsub -all -- "^# $moduleDatabaseConfig(header)" $firstline "" version
          if {![string compare [set version [string trim $version]] $moduleDatabaseConfig(version)]} then {
            close $fd
            return [arrayLoad moduleDatabaseData $configData(moddbfile) $verbose "module database file "]
          } elseif {[info exists moduleDatabaseFormat($version)]} then {
            if {[info exists moduleDatabaseData]} then {
              unset moduleDatabaseData
            }
            while {![eof $fd]} {
              set line [replaceExpr [gets $fd] "^ "]
              if {([string compare "" $line]) && \
                  (![regexp -- "^#" $line])} then {
                set moduleDatabaseData([lindex $line 0]) [dataFormatConvert [join $moduleDatabaseFormat($version)] [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] [lindex $line 1]]
              }
            }
            close $fd
            set moduleDatabaseDataChanged 0
            return 1
          } else {
            wpLog o * "Error: unknown module database version: $version"
          }
        } else {
          wpLog o * "Error: unknown module database format: [string trimleft $firstline " \t#"]"
        }
      }
      close $fd
    }
    return 0
  }

  ##
  ## Compare loaded module database data to module files
  ##
  ## Args:    verbose {-1,0,1}
  ## Returns: 1 if data matches,
  ##          0 otherwise
  ##
  proc compareModuleDatabase {{verbose 0}} {
    variable configData
    variable moduleDatabaseConfig

    foreach path $configData(modulepath) {
      set files [findFiles $path $moduleDatabaseConfig(maxdepth) .tcl]
      if {[string compare "" $files]} then {
        set comparedModules ""
        set moduleList [listModules]
        # Return early if we don't have any modules listed in the db
        if {![string compare "" $moduleList]} then {
          return 0
        }
        foreach file $files {
          set shortfile [string trimleft [string range $file [string length $path] end] /]
          if {$verbose >= 1} then {
            wpLog o * "Comparing file `$shortfile'"
          }
          if {[catch {set name [lindex [scanModule $file name] 0]} result]} then {
            if {$verbose >= 0} then {
              wpLog o * "Error: unable to open file `$shortfile' for reading: $result"
            }
          }
          # Process this file if it's a module
          if {[string compare "" $name]} then {
            # Get module filename from db
            set filename [getModuleDatabaseData $name file]
            # Compare module filename and make sure it wasn't renamed or moved
            if {[string compare $file $filename]} then {
              return 0
            }
            # Compare md5 from module db and sure the module hasn't changed
            if {([catch {set md5sum [md5Sum $filename]}]) || \
                ([string compare [getModuleDatabaseData $name md5sum] $md5sum])} then {
              return 0
            }
            # Append module name to list of compared modules
            lappend comparedModules $name
          }
        }
        # Compare list of compared modules with list of modules from the db
        if {[string compare [lsort $comparedModules] $moduleList]} then {
          return 0
        }
      }
    }
    return 1
  }

  ##
  ## Rebuild module database
  ##
  ## Args:    verbose {-1,0,1}
  ## Returns: nothing
  ##
  proc rebuildModuleDatabase {{verbose 0}} {
    variable configData
    variable moduleDatabaseConfig
    variable moduleDatabaseFormat
    variable moduleDatabaseData

    foreach path $configData(modulepath) {
      set files [findFiles $path $moduleDatabaseConfig(maxdepth) .tcl]
      set loadIndex [lsearch -exact [join $moduleDatabaseFormat([set moduleDatabaseConfig(version)])] load]
      foreach file $files {
        set shortfile [string trimleft [string range $file [string length $path] end] /]
        if {$verbose >= 1} then {
          wpLog o * "Scanning file `$shortfile'"
        }
        if {[catch {set data [scanModule $file]} result]} then {
          if {$verbose >= 0} then {
            wpLog o * "Warning: unable to open file `$shortfile' for reading: $result"
          }
        } else {
          set name [lindex $data 0]
          if {[string compare "" $name]} then {
            if {[moduleExists $name]} then {
              set tmp($name) [lreplace [lrange $data 1 end] $loadIndex $loadIndex [lindex $moduleDatabaseData($name) $loadIndex]]
            } else {
              set tmp($name) [lrange $data 1 end]
            }
          }
        }
      }
    }
    if {[info exists moduleDatabaseData]} then {
      unset moduleDatabaseData
    }
    if {[info exists tmp]} then {
      array set moduleDatabaseData [array get tmp]
    }
    return
  }

  ##
  ## List all modules in the database
  ##
  ## Args:    none
  ## Returns: list of modules in module database
  ##
  proc listModules {{loaded 0}} {
    variable moduleDatabaseData
    variable moduleLoadedList

    if {$loaded} then {
      return [lsort $moduleLoadedList]
    }
    return [lsort [array names moduleDatabaseData]]
  }

  ##
  ## Check if the given module exists
  ##
  ## Args:    module name
  ## Returns: 1 if the given module exists
  ##          0 otherwise
  ##
  proc moduleExists {module} {
    variable moduleDatabaseData

    if {[info exists moduleDatabaseData($module)]} then {
      return 1
    }
    return 0
  }

  ##
  ## Check if a module is loaded
  ##
  ## Args:    module name
  ## Returns: 1 if the given module is loaded
  ##          0 otherwise
  ##
  proc moduleLoaded {module} {
    variable moduleLoadedList

    if {[lsearch -exact $moduleLoadedList $module] != -1} then {
      return 1
    }
    return 0
  }

  ##
  ## Load a module
  ##
  ## Args:    module name, verbose {-1,0,1}, args {loop detection}
  ## Returns: nothing
  ## Errors:  if unable to load module
  ##
  proc moduleLoad {module {verbose 0} args} {
    variable NamespaceCurrent
    variable moduleLoadedList

    if {[lsearch -exact [set loop [lindex $args 0]] $module] == -1} then {
      if {[moduleExists $module]} then {
        set preload ""
        set requires [getModuleDatabaseData $module requires]
        foreach required $requires {
          set preloadModule [whichModuleCommand $required]
          if {[string compare "" $preloadModule]} then {
            if {([lsearch -exact $preload $preloadModule] == -1) && \
                ([string compare wp $preloadModule]) && \
                (![moduleLoaded $preloadModule])} then {
              lappend preload $preloadModule
            }
          } else {
            error "required command `$required' not found."
          }
        }
        if {[string compare "" $preload]} then {
          foreach premod $preload {
            if {[catch {moduleLoad $premod $verbose [concat $loop $module]} result]} then {
              error $result
            }
          }
        }
        if {[catch {source [getModuleDatabaseData $module file]} result]} then {
          error $result
        } else {
          package forget $module
          package provide ${NamespaceCurrent}::${module} [getModuleDatabaseData $module version]
          moduleConfig load $module 1
          moduleConfig checkdefs $module 1
          moduleData load $module 1
          # Imported commands '# requires: ...'
          if {[string compare "" $requires]} then {
            set Eval "namespace eval ${NamespaceCurrent}::${module} \{\n  namespace forget *\n  namespace import"
            foreach required $requires {
              if {[string compare "" [set command [whichCommand $required]]]} then {
                append Eval " $command"
              }
            }
            append Eval "\n\}"
            eval $Eval
          }
          # Exported commands '# provides: ...'
          if {[string compare "" [set provides [getModuleDatabaseData $module provides]]]} then {
            set Eval "namespace eval ${NamespaceCurrent}::${module} \{\n  namespace export"
            foreach provided $provides {
              append Eval " $provided"
            }
            append Eval "\n\}"
            eval $Eval
          }
          if {[string compare "" [info commands ${NamespaceCurrent}::${module}::InitTable]]} then {
            ${NamespaceCurrent}::${module}::InitTable load
          }
          if {[string compare "" [info commands ${NamespaceCurrent}::${module}::TraceTable]]} then {
            ${NamespaceCurrent}::${module}::TraceTable variable
          }
          if {[string compare "" [info commands ${NamespaceCurrent}::${module}::BindTable]]} then {
            ${NamespaceCurrent}::${module}::BindTable bind *
          }
          if {![getModuleDatabaseData $module load]} then {
            setModuleDatabaseData $module load 1
          }
          if {[lsearch -exact $moduleLoadedList $module] == -1} then {
            lappend moduleLoadedList $module
            if {$verbose >= 1} then {
              wpLog o * "Module loaded: $module"
            }
          }
        }
      } else {
        error "No such module: $module"
      }
    } else {
      regsub -all -- " " $loop " -> " loop
      error "Preload endless loop: $loop -> $module"
    }
    return
  }

  ##
  ## Unload a module
  ##
  ## Args:    module name, verbose {-1,0,1}
  ## Returns: nothing
  ## Errors:  if unable to completely unload module
  ##
  proc moduleUnload {module {verbose 0}} {
    variable NamespaceCurrent
    variable moduleLoadedList

    # FIXME: handle dependant modules and modules that can't be unloaded
    if {[moduleExists $module]} then {
      if {[string compare "" [info commands ${NamespaceCurrent}::${module}::InitTable]]} then {
        ${NamespaceCurrent}::${module}::InitTable unload
      }
      if {[string compare "" [info commands ${NamespaceCurrent}::${module}::BindTable]]} then {
        ${NamespaceCurrent}::${module}::BindTable unbind *
      }
      if {[string compare "" [info commands ${NamespaceCurrent}::${module}::TraceTable]]} then {
        ${NamespaceCurrent}::${module}::TraceTable vdelete
      }
      moduleConfig save $module 1
      moduleData save $module 1
      if {[catch {namespace delete ${NamespaceCurrent}::${module}} result]} then {
        error $result
      } else {
        package forget ${NamespaceCurrent}::${module}
        if {[getModuleDatabaseData $module load] == 1} then {
          setModuleDatabaseData $module load 0
        }
        set index [lsearch -exact $moduleLoadedList $module]
        if {$index != -1} then {
          set moduleLoadedList [lreplace $moduleLoadedList $index $index]
          if {$verbose >= 1} then {
            wpLog o * "Module unloaded: $module"
          }
        }
      }
    } else {
      error "No such module: $module"
    }
    return
  }

  ##
  ## Load / save module config data for a module
  ##
  ## Args:    action {load|save|checkdefs}, module, force {0,1},
  ##           verbose {-1,0,1}
  ## Returns: 1 if successful,
  ##          0 otherwise
  ##
  proc moduleConfig {action module {force 0} {verbose 0}} {
    variable NamespaceCurrent
    variable configData

    if {([string compare "" \
          [set file [getModuleDatabaseData $module config]]]) && \
        ([createDir $configData(configpath)])} then {
      set cfgfile [file join $configData(configpath) $file]
      switch -exact -- $action {
        save {
          if {([getModuleDatabaseData $module load]) && \
              (($force) || \
               (![info exists ${NamespaceCurrent}::${module}::configDataChanged]) || \
               ([set ${NamespaceCurrent}::${module}::configDataChanged]))} then {
            if {[arraySave ${NamespaceCurrent}::${module}::configData $cfgfile 0 "$module configuration file "]} then {
              if {$verbose >= 1} then {
                wpLog o * "Writing $module config file..."
              }
              set ${NamespaceCurrent}::${module}::configDataChanged 0
              return 1
            } elseif {$verbose >= 0} then {
              wpLog o * "Error writing $module config file."
            }
          }
        }
        load {
          if {($force) || \
              (![info exists ${NamespaceCurrent}::${module}::configDataChanged]) || \
              (![set ${NamespaceCurrent}::${module}::configDataChanged])} then {
            if {[arrayLoad ${NamespaceCurrent}::${module}::configData $cfgfile 0 "$module configuration file "]} then {
              if {$verbose >= 1} then {
                wpLog o * "Loading $module config file..."
              }
              set ${NamespaceCurrent}::${module}::configDataChanged 0
              return 1
            } elseif {$verbose >= 0} then {
              wpLog o * "Error loading $module config file."
            }
          }
        }
        checkdefs {
          if {[array exists ${NamespaceCurrent}::${module}::configDataDefaults]} then {
            set Changed 0
            # Unset unknown variables
            foreach name [array names ${NamespaceCurrent}::${module}::configData] {
              if {![info exists ${NamespaceCurrent}::${module}::configDataDefaults($name)]} then {
                unset ${NamespaceCurrent}::${module}::configData($name)
                set Changed 1
              }
            }
            # Set missing variables to defaults
            foreach {name data} [array get ${NamespaceCurrent}::${module}::configDataDefaults] {
              if {![info exists ${NamespaceCurrent}::${module}::configData($name)]} then {
                set ${NamespaceCurrent}::${module}::configData($name) $data
                set Changed 1
              }
            }
            if {$Changed} then {
              set ${NamespaceCurrent}::${module}::configDataChanged 1
            }
            return 1
          }
        }
      }
    }
    return 0
  }

  ##
  ## Load / save module config data for a list of modules
  ##
  ## Args:    action {load|save}, module list, force {0,1}, verbose {-1,0,1}
  ## Returns: nothing
  ##
  proc moduleConfigList {action modules {force 0} {verbose 0}} {
    variable moduleDatabaseData

    if {![string compare * $modules]} then {
      set modules [listModules 1]
    }
    foreach module $modules {
      if {([moduleExists $module]) && ([moduleLoaded $module])} then {
        moduleConfig $action $module $force $verbose
      }
    }
    return
  }

  ##
  ## Load / save module data for a module
  ##
  ## Args:    action {load|save|backup}, module, force {0,1},
  ##           verbose {-1,0,1}
  ## Returns: nothing
  ##
  proc moduleData {action module {force 0} {verbose 0}} {
    variable NamespaceCurrent
    variable configData

    if {([info exists ${NamespaceCurrent}::${module}::dataOptions]) && \
        ([createDir $configData(datapath)])} then {
      switch -exact -- $action {
        save {
          foreach {name data} [array get ${NamespaceCurrent}::${module}::dataOptions] {
            foreach {type file desc} $data {break}
            if {([info exists type]) && ([info exists file]) && \
                ([info exists desc])} then {
              set Changed ${NamespaceCurrent}::${module}::${name}Changed
              if {($force) || ((![info exists $Changed]) || ([set $Changed]))} then {
                if {[${type}Save ${NamespaceCurrent}::${module}::$name [file join $configData(datapath) $file]]} then {
                  if {$verbose >= 1} then {
                    wpLog o * $NamespaceCurrent "Writing $desc data file..."
                  }
                  set $Changed 0
                } elseif {$verbose >= 0} then {
                  wpLog o * $NamespaceCurrent "Error writing $desc data file!"
                }
              }
            }
          }
        }
        load {
          foreach {name data} [array get ${NamespaceCurrent}::${module}::dataOptions] {
            foreach {type file desc} $data {break}
            if {([info exists type]) && ([info exists file]) && \
                ([info exists desc])} then {
              set Changed ${NamespaceCurrent}::${module}::${name}Changed
              if {($force) || ((![info exists $Changed]) || ([set $Changed]))} then {
                if {[${type}Load ${NamespaceCurrent}::${module}::$name [file join $configData(datapath) $file]]} then {
                  if {$verbose >= 1} then {
                    wpLog o * $NamespaceCurrent "Reloading $desc data file..."
                  }
                  set $Changed 0
                } elseif {$verbose >= 0} then {
                  wpLog o * $NamespaceCurrent "Error reloading $desc data file!"
                }
              }
            }
          }
        }
        backup {
          foreach {name data} [array get ${NamespaceCurrent}::${module}::dataOptions] {
            foreach {type file desc} $data {break}
            if {([info exists type]) && ([info exists file]) && \
                ([info exists desc])} then {
              if {[set result [backupFile [file join $configData(datapath) $file] $verbose]]} then {
                if {($result >= 1) && ($verbose >= 1)} then {
                  wpLog o * $NamespaceCurrent "Backing up $desc data file..."
                }
              } elseif {$verbose >= 0} then {
                wpLog o * $NamespaceCurrent "Error backing up $desc data file!"
              }
            }
          }
        }
      }
    }
    return
  }

  ##
  ## Load / save module data for a list of modules
  ##
  ## Args:    action {load|save}, module list, force {0,1}, verbose {-1,0,1}
  ## Returns: nothing
  ##
  proc moduleDataList {action modules {force 0} {verbose 0}} {
    variable moduleDatabaseData

    if {![string compare * $modules]} then {
      set modules [listModules 1]
    }
    foreach module $modules {
      if {([moduleExists $module]) && ([moduleLoaded $module])} then {
        moduleData $action $module $force $verbose
      }
    }
    return
  }

  ##
  ## Builds command matching table from module database
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc buildCommandTable {{verbose 0}} {
    variable NamespaceCurrent
    variable ExportList
    variable commandTable

    foreach command $ExportList {
      if {![info exists tmp($command)]} then {
        if {$verbose >= 2} then {
          wpLog o * "Adding command: `$command' as `${NamespaceCurrent}::$command'"
        }
        set tmp($command) ${NamespaceCurrent}::$command
      } elseif {$verbose >= 0} then {
        wpLog o * "Warning: ignoring duplicate command `$command', conflicts with `$tmp($command)'"
      }
    }
    foreach module [listModules] {
      if {[string compare "" [set provides [getModuleDatabaseData $module provides]]]} then {
        foreach command $provides {
          if {![info exists tmp($command)]} then {
            if {$verbose >= 2} then {
              wpLog o * "Adding command: `$command' as `${NamespaceCurrent}::${module}::$command'"
            }
            set tmp($command) ${NamespaceCurrent}::${module}::$command
          } elseif {$verbose >= 0} then {
            wpLog o * "Warning: ignoring duplicate command `$command', conflicts with `$tmp($command)'"
          }
        }
      }
    }
    if {[info exists commandTable]} then {
      unset commandTable
    }
    array set commandTable [array get tmp]
    return
  }

  ##
  ## Return full namespace path for the given command
  ##
  ## Args:    command
  ## Returns: full namespace path for the given command if it exists
  ##          nothing otherwise
  ##
  proc whichCommand {command} {
    variable commandTable

    if {[info exists commandTable($command)]} then {
      return $commandTable($command)
    }
    return
  }

  ##
  ## Return full namespace path for the given module
  ##
  ## Args:    module
  ## Returns: full namespace path for the given module if it's loaded
  ##          nothing otherwise
  ##
  proc whichModule {module} {
    variable NamespaceCurrent

    if {![string compare $module [namespace tail $NamespaceCurrent] $module]} then {
      return $NamespaceCurrent
    } elseif {[moduleLoaded $module]} then {
      return ${NamespaceCurrent}::$module
    }
    return
  }

  ##
  ## Return module name that provides the given command
  ##
  ## Args:    command
  ## Returns: name of module that provides the given command
  ##          nothing otherwise
  ##
  proc whichModuleCommand {command} {
    variable NamespaceCurrent
    variable commandTable

    if {[info exists commandTable($command)]} then {
      if {![string compare ${NamespaceCurrent}::$command \
            $commandTable($command)]} then {
        return [namespace tail $NamespaceCurrent]
      }
      return [namespace tail [namespace qualifiers $commandTable($command)]]
    }
    return
  }

  ##
  ## Compare the given version to eggdrop's version
  ##
  ## Args:    version
  ## Returns: 0 if eggdrop's version is older then the given version
  ##          1 if eggdrop's version matches the given version
  ##          2 if eggdrop's version is newer then the given version
  ##
  proc compareVersion {version} {
    global numversion

    if {[string compare "" $version]} then {
      if {([info exists numversion]) || \
          (![catch {set numversion}]) || \
          ([info exists numversion])} then {
        if {[regexp -- \\. $version]} then {
          regsub -all -- \\. $version 0 version
          set version ${version}00
        }
        if {![regexp -- \[^0-9\] $version]} then {
          if {$numversion > $version} then {
            return 2
          } elseif {$numversion == $version} then {
            return 1
          }
        }
      }
    }
    return 0
  }

  ##
  ## Log module information
  ##
  ## Args:    level, channel, args ({<namespace>} {<text>} | {<text>})
  ## Returns: nothing
  ##
  proc wpLog {level channel args} {
    if {[llength $args] == 2} then {
      if {[string compare wp [set namespace [namespace tail [lindex $args 0]]]]} then {
        putloglev $level $channel "Wolfpack: \[$namespace\] [lindex $args 1]"
      } else {
        putloglev $level $channel "Wolfpack: [lindex $args 1]"
      }
    } else {
      putloglev $level $channel "Wolfpack: [join $args]"
    }
    return
  }

  ##
  ## Evaluate command line arguments
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc EvalArgs {argc argv argv0} {
    variable NamespaceCurrent
    variable optionData

    # Make sure defaults are sane
    arraySetAll optionData 0
    for {set index 0} {$index < $argc} {incr index} {
      set option [lindex $argv $index]
      switch -regexp -- $option {
        (^--config$) {
          set optionData(config) 1
        }
        (^--rebuild$) {
          set optionData(rebuild) 1
        }
        (^--norebuild$) {
          set optionData(norebuild) 1
        }
        (^--time$) {
          set optionData(time) 1
        }
        (^--verbose$) {
          incr optionData(verbose)
        }
        (^--quiet$) {
          incr optionData(quiet) -1
        }
        (^--debug$) {
          set optionData(debug) 1
        }
        (^--help$) {
          ShowUsage $argv0
          exit
        }
        (^--version$) {
          puts "[file tail $argv0] version [package require ${NamespaceCurrent}]"
          exit
        }
        (^-\[^-\]*$) {
          set suboptions [split $option ""]
          set sublength [llength [split $suboptions]]
          for {set subindex 0} {$subindex < $sublength} {incr subindex} {
            set suboption [lindex $suboptions $subindex]
            switch -exact -- $suboption {
              - {
                continue
              }
              c {
                set optionData(config) 1
              }
              r {
                set optionData(rebuild) 1
              }
              n {
                set optionData(norebuild) 1
              }
              t {
                set optionData(time) 1
              }
              v {
                incr optionData(verbose)
              }
              q {
                incr optionData(quiet) -1
              }
              d {
                set optionData(debug) 1
              }
              default {
                if {(![info exists invalidopt]) || \
                    ($subindex == 1) || \
                    ([lsearch -exact $invalidopt $option] == -1)} then {
                  lappend invalidopt $option
                }
              }
            }
          }
        }
        default {
          lappend invalidopt $option
        }
      }
    }

    # complain about invalid command line arguments
    if {[info exists invalidopt]} then {
      foreach option $invalidopt {
        puts stderr "[file tail $argv0]: unrecognized option `$option'"
      }
      exit 1
    }
  }

  ##
  ## Show usage information
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc ShowUsage {argv0} {
    # FIXME: code missing options
    puts "Usage: [file tail $argv0] <options>"
    puts "  Valid options:"
    puts "  -c, --config         start interactive configuration"
    #puts "  -u, --update         update module database"
    puts "  -r, --rebuild        force rebuild of module database"
    puts "  -n, --norebuild      don't rebuild module database even if it's outdated"
    puts "  -t, --time           time compare and rebuild of module database"
    #puts "  -i, --include <mod>  include `module' when building database"
    #puts "  -e, --exclude <mod>  exclude `module' when building database"
    #puts "  -m, --module <mod>   only update database for `module'"
    puts "  -v, --verbose        use more than once for more verbose operation"
    puts "  -q, --quiet          use more than once for quieter operation"
    puts "  -d, --debug          start in debug mode with tclsh"
    puts "      --help           show this help"
    puts "      --version        show version information"
  }

  ##
  ## Enter interactive configuration
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc Iconfig {} {
    variable NamespaceCurrent
    variable configDefaults

    fileevent stdin readable ${NamespaceCurrent}::IconfigReadStdin
    puts "Entering wolfpack configuration system..."
    puts "Type 'help' for help."
    puts -nonewline $configDefaults(prompt)
    flush stdout
    vwait forever
  }

  ##
  ## Read stdin and process commands
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc IconfigReadStdin {} {
    variable configDefaults
    variable configData

    set exit 0
    if {[eof stdin]} {
      set exit 1
    }
    set stdin [string trimright [gets stdin]]
    set data [join [lrange [split $stdin] 1 end]]
    if {[string compare "" $stdin]} then {
      switch -exact -- [lindex [split $stdin] 0] {
        set {
          IconfigSet $data
        }
        enable {
          IconfigEnable $data
        }
        disable {
          IconfigDisable $data
        }
        modules {
          IconfigModules $data
        }
        help {
          IconfigHelp $data
        }
        quit {
          set exit 1
        }
        default {
          puts "What?  You need 'help'"
        }
      }
    }
    if {(!$exit) && (![eof stdin])} then {
      puts -nonewline $configDefaults(prompt)
      flush stdout
    } else {
      # Save configuration data
      arraySave configData $configDefaults(cfgfile) 0 "configuration file "
      # Save module database
      saveModuleDatabase
      puts ""
      flush stdout
      exit
    }
    return
  }

  ##
  ## Set configuration settings
  ##
  ## Args:    data
  ## Returns: nothing
  ##
  proc IconfigSet {data} {
    variable configData
    variable configDataDesc

    if {![string compare "" $data]} then {
      set fmtlen1 [arrayMaxElementDataLength configDataDesc]
      if {$fmtlen1 < 13} then {
        set fmtlen1 13 ;# 'Description: '
      }
      set names [array names configData]
      set fmtlen2 [listMaxElementLength $names]
      if {$fmtlen2 < 8} then {
        set fmtlen2 8 ;# 'Option: '
      }
      puts ""
      puts "Current settings:"
      puts ""
      puts [format "  %-${fmtlen1}s    %-${fmtlen2}s    %s" Description: Option: Value:]
      foreach option [lsort $names] {
        if {[info exists configDataDesc($option)]} then {
          puts [format "  %-${fmtlen1}s    %-${fmtlen2}s    %s" $configDataDesc($option) $option $configData($option)]
        } else {
          puts [format "  %-${fmtlen1}s    %-${fmtlen2}s    %s" "" $option $configData($option)]
        }
      }
      puts ""
    } else {
      set option [lindex [split $data] 0]
      if {![info exists configData($option)]} then {
        puts "Invalid option: $option"
      } else {
        set value [join [lrange [split $data] 1 end]]
        if {![string compare "" $value]} then {
          puts "Currently: $configData($option)"
        } else {
          set configData($option) $value
          puts "Set $option to: $value"
        }
      }
    }
    return
  }

  ##
  ## Enable a module
  ##
  ## Args:    data
  ## Returns: nothing
  ##
  proc IconfigEnable {data} {
    set module [lindex [split $data] 0]
    if {![string compare "" $module]} then {
      puts "Usage: enable <module>"
    } elseif {[moduleExists $module]} then {
      if {![getModuleDatabaseData $module load]} then {
        setModuleDatabaseData $module load 1
        puts "Enabled module: $module"
      } else {
        puts "Module `$module' is already enabled."
      }
    } else {
      puts "Invalid module: $module"
    }
    return
  }

  ##
  ## Disable a module
  ##
  ## Args:    data
  ## Returns: nothing
  ##
  proc IconfigDisable {data} {
    set module [lindex [split $data] 0]
    if {![string compare "" $module]} then {
      puts "Usage: disable <module>"
    } elseif {[moduleExists $module]} then {
      if {[getModuleDatabaseData $module load] == 1} then {
        setModuleDatabaseData $module load 0
        puts "Disabled module: $module"
      } else {
        puts "Module `$module' is already disabled."
      }
    } else {
      puts "Invalid module: $module"
    }
    return
  }

  ##
  ## List modules
  ##
  ## Args:    data
  ## returns: nothing
  ##
  proc IconfigModules {data} {
    puts "Modules avaliable:"
    foreach module [listModules] {
      puts "$module"
      foreach line [splitList [getModuleDatabaseData $module description] 65 " " " "] {
        puts "  $line"
      }
    }
    return
  }

  ##
  ## Show configuration help
  ##
  ## Args:    data
  ## Returns: nothing
  ##
  proc IconfigHelp {data} {
    if {![string compare "" $data]} then {
      set data help
    }
    switch -exact -- $data {
      set {
        puts "###  set \[option\] \[value\]"
        puts "   Sets an option to what you specify."
        puts "   Shows current setting for an option if no new value is given."
        puts "   Shows current settings for all options if no option is given."
      }
      enable {
        puts "###  enable <module>"
        puts "   Enables the given module."
      }
      disable {
        puts "###  disable <module>"
        puts "   Disables the given module."
      }
      modules {
        puts "###  modules"
        puts "   Shows modules avaliable."
      }
      help {
        puts "###  Avaliable commands:"
        puts "  set     \[option\] \[value\]"
        puts "  enable  <module>"
        puts "  disable <module>"
        puts "  modules"
        puts "  help    \[command\]"
        puts "  quit"
        puts "You can get help on individual commands: 'help <command>'"
      }
      quit {
        puts "###  quit"
        puts "   Quits interactive configuration."
      }
      default {
        puts "No help available on that."
      }
    }
    return
  }

  ##
  ## Inline startup and init code
  ##

  wpLog o * "wolfpack.tcl v[package require $NamespaceCurrent] loading..."

  # Init md5Sum command
  if {![md5Init]} then {
    wpLog o * "Error: can't find a usable md5 command!"
    return 0
  }

  # Export commands
  eval namespace export [join $ExportList]

  # Set missing variables to default values
  if {![info exists configDataChanged]} then {
    set configDataChanged 0
  }
  if {![info exists moduleDatabaseDataChanged]} then {
    set moduleDatabaseDataChanged 0
  }
  if {![info exists moduleLoadedList]} then {
    set moduleLoadedList ""
  }

  # Load configuration data
  arrayLoad configData $configDefaults(cfgfile) 0 "configuration file "

  # Unset unknown configuration variables
  foreach name [array names configData] {
    if {![info exists configDataDefaults($name)]} then {
      unset configData($name)
      set configDataChanged 1
    }
  }

  # Set missing configuration variables to defaults
  foreach {name data} [array get configDataDefaults] {
    if {![info exists configData($name)]} then {
      set configData($name) $data
      set configDataChanged 1
    }
  }

  # Save configuration data if changed
  if {$configDataChanged} then {
    arraySave configData $configDefaults(cfgfile) 0 "configuration file "
  }

  # Eval command line arguments if loading with tclsh
  if {[info exists argv0]} then {
    EvalArgs $argc $argv $argv0
  }

  # Check verbose/quiet options
  if {$optionData(quiet)} then {
    set Verbose $optionData(quiet)
  } elseif {$configData(verbose)} then {
    set Verbose $configData(verbose)
  } else {
    set Verbose $optionData(verbose)
  }

  # Check rebuild/norebuild options
  if {(!$optionData(norebuild)) && ($optionData(rebuild))} then {
    set RebuildDatabase 1
  } else {
    set RebuildDatabase 0
  }

  # Sanity check: old eggdrop versions used [time] for a timestamp command
  if {(($optionData(time)) || ($configData(time))) && \
      ([catch {time}])} then {
    set TimeOk 1
  } else {
    set TimeOk 0
  }

  # Load module database
  if {$Verbose >= 0} then {
    wpLog o * "Loading module database..."
  }

  if {$TimeOk} then {
    set LoadTime [time {set LoadResult [loadModuleDatabase $Verbose]}]
  } else {
    set LoadResult [loadModuleDatabase $Verbose]
  }

  if {$LoadResult != -1} then {
    if {$Verbose >= 0} then {
      if {[info exists LoadTime]} then {
        wpLog o * "Done. ([format "%.3f" [expr [lindex $LoadTime 0] / 1000000.0]] seconds elapsed)"
      } else {
        wpLog o * "Done."
      }
    }
    set CreateDatabase 0
  } else {
    if {$Verbose >= 0} then {
      wpLog o * "Warning: module database does not exist."
    }
    set CreateDatabase 1
  }

  set NeedsRebuild 0

  # Compare module database if we are not going to rebuild or create it
  if {(!$RebuildDatabase) && (!$CreateDatabase)} then {
    if {$Verbose >= 0} then {
      wpLog o * "Comparing module database..."
    }
    if {![file exists $configData(moddbfile)]} then {
      set NeedsRebuild 1
    } else {
      if {$TimeOk} then {
        set CompareTime [time {set CompareResult [compareModuleDatabase $Verbose]}]
      } else {
        set CompareResult [compareModuleDatabase $Verbose]
      }
      if {!$CompareResult} then {
        set NeedsRebuild 1
      }
    }

    if {$Verbose >= 0} then {
      if {[info exists CompareTime]} then {
        wpLog o * "Done. ([format "%.3f" [expr [lindex $CompareTime 0] / 1000000.0]] seconds elapsed)"
      } else {
        wpLog o * "Done."
      }
      if {$NeedsRebuild} then {
        wpLog o * "Database is outdated."
      } else {
        wpLog o * "Database is current."
      }
    }
  }

  # Create database if does not exist
  # Rebuild database if requested
  # Rebuild database if it's outdated and config permits
  if {($CreateDatabase) || ($RebuildDatabase) || \
      (($NeedsRebuild) && ($configData(rebuild)))} then {

    if {$Verbose >= 0} then {
      if {$CreateDatabase} then {
        wpLog o * "Creating module database..."
      } else {
        wpLog o * "Rebuilding module database..."
      }
    }

    # Rebuild module database
    if {$TimeOk} then {
      set RebuildTime [time {rebuildModuleDatabase $Verbose}]
    } else {
      rebuildModuleDatabase $Verbose
    }

    if {$Verbose >= 0} then {
      if {[info exists RebuildTime]} then {
        wpLog o * "Done. ([format "%.3f" [expr [lindex $RebuildTime 0] / 1000000.0]] seconds elapsed)"
      } else {
        wpLog o * "Done."
      }
    }

    # Save module database
    if {$Verbose >= 0} then {
      wpLog o * "Saving module database..."
    }

    if {$TimeOk} then {
      set SaveTime [time {set SaveResult [saveModuleDatabase $Verbose]}]
    } else {
      set SaveResult [saveModuleDatabase $Verbose]
    }

    if {$Verbose >= 0} then {
      if {$SaveResult} then {
        if {[info exists SaveTime]} then {
          wpLog o * "Done. ([format "%.3f" [expr [lindex $SaveTime 0] / 1000000.0]] seconds elapsed)"
        } else {
          wpLog o * "Done."
        }
      } else {
        wpLog o * "Error"
      }
    }

  } elseif {($NeedsRebuild) && ($Verbose >= 0)} then {
    wpLog o * "Warning: not rebuilding outdated module database..."
  }

  # Enter interactive configuration if loading with tclsh
  if {[info exists argv0]} then {
    if {$optionData(config)} then {
      Iconfig
    }
  }

  # Build command table
  if {(![info exists argv0]) || ($optionData(debug))} then {
    if {$Verbose >= 0} then {
      wpLog o * "Building command table..."
    }
    if {$TimeOk} then {
      set CommandTime [time {buildCommandTable $Verbose}]
    } else {
      buildCommandTable $Verbose
    }
    if {$Verbose >= 0} then {
      if {[info exists CommandTime]} then {
        wpLog o * "Done. ([format "%.3f" [expr [lindex $CommandTime 0] / 1000000.0]] seconds elapsed)"
      } else {
        wpLog o * "Done."
      }
    }

    set loadList ""

    # Load debug module first if not in eggdrop mode
    if {([info exists argv0]) && ([moduleExists debug])} then {
      lappend loadList debug
    }

    # Load other modules next, moduleLoad will preload additional modules
    foreach name [lsort [array names moduleDatabaseData]] {
      if {[getModuleDatabaseData $name load] == 1} then {
        lappend loadList $name
      }
    }

    wpLog o * "Loading modules..."

    # Load modules
    if {$TimeOk} then {
      set LoadTime [time {
        foreach module $loadList {
          if {[catch {moduleLoad $module $Verbose} result]} then {
            wpLog o * "Error: unable to load module `$module': $result"
          }
        }
      }]
    } else {
      foreach module $loadList {
        if {[catch {moduleLoad $module $Verbose} result]} then {
          wpLog o * "Error: unable to load module `$module': $result"
        }
      }
    }

    if {[info exists LoadTime]} then {
      wpLog o * "Done. ([format "%.3f" [expr [lindex $LoadTime 0] / 1000000.0]] seconds elapsed)"
    } else {
      wpLog o * "Done."
    }

    set ModuleLoadCount [llength [split $moduleLoadedList]]
    set ModuleTotalCount [llength [array names moduleDatabaseData]]
    if {$ModuleLoadCount} then {
      wpLog o * "Modules loaded ($ModuleLoadCount/$ModuleTotalCount): $moduleLoadedList"
    } else {
      wpLog o * "No modules loaded."
    }

    # Unload modules if started in debug mode
    if {$optionData(debug)} then {

# FIXME: code module dependency stuff first
#      wpLog o * "Unloading modules..."

      # Unload all modules except debug
#      foreach module $moduleLoadedList {
#        if {[string compare $module debug]} then {
#          if {[catch {moduleUnload $module $Verbose} result]} then {
#            wpLog o * "Error: unable to unload module `$module': $result"
#          }
#        }
#      }

      # Unload debug module last to make it display stats
      if {[moduleLoaded debug]} then {
        if {[catch {moduleUnload debug} result]} then {
          wpLog o * "Error: unable to unload module `debug': $result"
        }
      }
#      wpLog o * "Done."
    }
  }

} ;# namespace ::wp

return
